/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

function syncCalendar(syncManager, folderId) {
	FW_LOG("syncCalendar folder=" + folderId + ", syncManager=" + syncManager);
	syncManager.remoteHandler = new calRemoteHandler();

	var ox6Handler = new calOX6RemoteHandler(syncManager);
	var connection = syncManager.connectionDB;

	syncManager.remoteHandler.registerHandler("OX6", ox6Handler);

	FW_LOG("set folder property for remote handler: " + folderId);
	ox6Handler.calendarFolder = folderId;

	FW_LOG("set sync manager server type: " + syncManager.serverType);
	syncManager.remoteHandler.setHandler(syncManager.serverType);

	FW_LOG("try to login with user: " + syncManager.USER);

	var sessionHandler = new SessionHandler(syncManager.HOST, syncManager.USER, syncManager.PASSWORD, syncManager.connectionDB);
	var newestSession = getNewestSession(connection);
	sessionHandler.setSession(newestSession[0], newestSession[1]);
	sessionHandler.validateSession(sessionCheckReady, [ syncManager, ox6Handler ], false);
}

function sessionCheckReady(sessionHandler, params, success) {
	var syncManager = params[0];
	if (success) {
		var ox6Handler = params[1];
		ox6Handler.session = sessionHandler.session;
		ox6Handler.cookie = sessionHandler.cookie;

		// call http request not as async for TB3 because it is already threaded

		syncManager.remoteHandler.readAppointments(syncManager.HOST, readAppointmentsReady, false);

	} else {
		FW_LOG("session check while reading appointments failed, try to read next folder");
		syncManager.startCalendarSync();
	}
}

function readAppointmentsFailed(folderId, syncManager) {
	FW_LOG("reading appointment for folder " + folderId + " failed, try to read next folder");
	syncManager.startCalendarSync();
}

function readAppointmentsReady(folderId, appointment_array, syncManager) {
	FW_LOG("readAppointmentsReady event detected for folder: " + folderId);
	if (!appointment_array || appointment_array.length == 0) {
		FW_LOG("folder " + folderId + " return 0 appointments");
		syncManager.startCalendarSync();
		return;
	}

	FW_LOG(appointment_array.length + " appointments fetched");

	var oldLocalIdArray = listAppLocalIdsForFolder(syncManager.connectionDB, folderId);
	var newLocalIdArray = [];

	var syncObjects = getCalendarSyncObjects(syncManager.connectionDB, folderId);
	var syncObjectsByRemoteId = convertSyncObjects2Map(syncObjects);

	var rangeStartAsLong = getPrefRangeStartAsLong(syncManager.connectionDB);
	var rangeEndAsLong = getPrefRangeEndAsLong(syncManager.connectionDB);

	syncManager.connectionDB.beginTransaction();

	var a = 0;

	var new_appointment_array = [];
	for (a = 0; a < appointment_array.length; a++) {
		var appointmentObj = appointment_array[a];

		if (!isAppointmentInRange(appointmentObj, rangeStartAsLong, rangeEndAsLong)) {
			continue;
		}

		var objectId = appointmentObj['object_id'];
		var syncObject = syncObjectsByRemoteId[objectId];

		var localId = appointmentObj['uuid'];

		if (!syncObject) {
			FW_LOG("local object doesn't exists, create new mapping");

			if (!localId) {
				localId = getUUID();
			}

			insertCalendarIdMapping(syncManager.connectionDB, folderId, localId, 0, objectId, 0, 0);
			syncObject = {
				'local_id' : localId,
				'local_timestamp' : 0,
				'remote_id' : objectId,
				'remote_folder' : folderId,
				'remote_timestamp' : 0
			};
		} else {
			if (!localId) {
				localId = syncObject['local_id'];
			}

			FW_LOG("mapping for local id " + localId + " found by remote id: " + objectId);
		}

		var startAndEnd = parseStartAndEndFromAppointment(appointmentObj);
		var start = startAndEnd[0];
		var end = startAndEnd[1];

		var appointmentObjAsString = FXJSON.toString(appointmentObj);

		FW_LOG("save appointment as pos " + a + " from " + appointment_array.length + " appointments");
		saveAppointmentData(syncManager.connectionDB, folderId, localId, start, end, appointmentObjAsString);

		syncManager.iCacheManager.deleteItemFromCache(localId);

		newLocalIdArray.push(localId);
		new_appointment_array.push(appointmentObj);
	}

	FW_LOG("read " + appointment_array.length + " appointments from server, added " + new_appointment_array.length + " appointments to local storage");
	// FW_LOG("newLocalIDArray: " + newLocalIdArray.toSource());

	// remove remote deleted id from local id mapping
	var deleteOldLocalIdEntries = [];
	for (a = 0; a < oldLocalIdArray.length; a++) {
		var oldLocalId = oldLocalIdArray[a];
		var found = false;
		for ( var b = 0; b < newLocalIdArray.length; b++) {
			if (oldLocalId == newLocalIdArray[b]) {
				found = true;
				break;
			}
		}

		if (!found) {
			deleteOldLocalIdEntries.push(oldLocalId);
		}
	}

	FW_LOG("delete " + deleteOldLocalIdEntries.length + " item from local storage");

	var deletedEntriesCount = deleteOldLocalIdEntries.length;
	if (deletedEntriesCount > 0) {
		FW_LOG(deletedEntriesCount + ' old local entries found to delete in folder ' + folderId);

		// delete only 50 elements at per on DB
		var maxElements = 50;

		while (deletedEntriesCount > 0) {
			var sliceEnd = deletedEntriesCount;
			if (sliceEnd > maxElements) {
				sliceEnd = maxElements;
			}

			var slicedArray = deleteOldLocalIdEntries.slice(0, sliceEnd);

			deleteAppointmentsData(syncManager.connectionDB, folderId, slicedArray);
			deleteCalendarIdMapping(syncManager.connectionDB, folderId, slicedArray);

			// remove first entries until sliceEnd
			for ( var i = 0; i < sliceEnd; i++) {
				deleteOldLocalIdEntries.shift();
			}

			deletedEntriesCount = deleteOldLocalIdEntries.length;
		}
	} else {
		FW_LOG("no old local entries to delete in folder " + folderId);
	}

	FW_LOG("merged " + new_appointment_array.length + " items to storage for folder " + folderId);

	syncManager.connectionDB.commitTransaction();

	syncManager.startCalendarSync();
}

function convertSyncObjects2Map(syncObjects) {
	var retval = {};

	for ( var a = 0; a < syncObjects.length; a++) {
		var syncObject = syncObjects[a];
		var remoteId = syncObject['remote_id'];

		retval[remoteId] = syncObject;
	}

	return retval;
}

function isAppointmentInRange(appointmentObj, rangeStartAsLong, rangeEndAsLong) {
	if (rangeStartAsLong) {
		// check until
		var endDateAsLong;
		if (appointmentObj['until']) {
			endDateAsLong = parseInt(appointmentObj['until']);
		} else {
			endDateAsLong = parseInt(appointmentObj['end_date']);
		}

		if (endDateAsLong < rangeStartAsLong) {
			FW_LOG("appointment " + appointmentObj.title + " is not in the given range, ignoring");
			return false;
		}
	}

	if (rangeEndAsLong) {
		var startDateAsLong = parseInt(appointmentObj['start_date']);

		if (startDateAsLong > rangeEndAsLong) {
			FW_LOG("appointment " + appointmentObj.title + " is not in the given range, ignoring");
			return false;
		}
	}

	return true;
}
