/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

var iDatabaseManager = Components.interfaces.IDatabaseManager;
var nsISupports = Components.interfaces.nsISupports;

function DatabaseManager() {
	// If you only need to access your component from Javascript, uncomment the
	// following line:
	this.wrappedJSObject = this;
}

// class definition
DatabaseManager.prototype = {

	initializeDatabase : function() {
		var con = getOXCalendarDBConnection();
		checkTables(con);
	},

	QueryInterface : function(aIID) {
		if (!aIID.equals(iDatabaseManager) && !aIID.equals(nsISupports)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}
		return this;
	}
}

function getOXCalendarDBConnection() {
	var file = Components.classes["@mozilla.org/file/directory_service;1"]
			.getService(Components.interfaces.nsIProperties).get("ProfD",
					Components.interfaces.nsIFile);

	file.append("oxcalendar.sqlite");

	var storageService = Components.classes["@mozilla.org/storage/service;1"]
			.getService(Components.interfaces.mozIStorageService);

	FW_LOG("try to open database");
	var connection = storageService.openDatabase(file);

	if (!connection.connectionReady) {
		FW_WARN("newly created connection is not ready");
	}

	return connection;
}

function checkTables(connection) {
	var prefProperties = {
		'name' : 'VARCHAR(70)',
		'value' : 'VARCHAR(70)'
	};

	var calendarIdMappingProperties = {
		'folder' : 'INTEGER',
		'local_id' : 'VARCHAR(50)',
		'local_timestamp' : 'VARCHAR(20)',
		'remote_id' : 'VARCHAR(20)',
		'remote_folder' : 'VARCHAR(20)',
		'remote_timestamp' : 'VARCHAR(20)'
	};

	var contactIdMappingProperties = {
		'storage_id' : 'INTEGER',
		'local_id' : 'VARCHAR(50)',
		'local_timestamp' : 'VARCHAR(20)',
		'remote_id' : 'VARCHAR(20)',
		'remote_folder' : 'VARCHAR(20)',
		'remote_timestamp' : 'VARCHAR(20)'
	};

	var subscribedFoldersProperties = {
		'calendar_id' : 'INTEGER',
		'id' : 'VARCHAR(20)',
		'module' : 'VARCHAR(15)'
	};

	var folderProperties = {
		'id' : 'VARCHAR(20)',
		'created_by' : 'VARCHAR(20)',
		'name' : 'VARCHAR(255)',
		'module' : 'VARCHAR(15)',
		'type' : 'VARCHAR(20)',
		'timestamp' : 'VARCHAR(20)'
	};

	var participantsMapping = {
		'calendar_id' : 'INTEGER',
		'id' : 'VARCHAR(40)',
		'display_name' : 'VARCHAR(70)',
		'mail' : 'VARCHAR(70)',
		'type' : 'INTEGER'
	};

	var addressbookMapping = {
		'addressbook_id' : 'INTEGER PRIMARY KEY AUTOINCREMENT',
		'addressbook_uri' : 'VARCHAR(50)'
	};

	var sessionStorage = {
		'id' : 'INTEGER PRIMARY KEY AUTOINCREMENT',
		'session' : 'VARCHAR(255)',
		'random' : 'VARCHAR(255)'
	};

	var appointmentData = {
		'folder' : 'INTEGER',
		'local_id' : 'VARCHAR(50)',
		'start' : 'INTEGER',
		'end' : 'INTEGER',
		'data' : 'TEXT'
	};

	checkOldTables(connection);

	// system tables
	checkTable(connection, "oxcalendar_preferences", prefProperties);
	// dropTable(connection, "oxcalendar_subscribed_folders");
	checkTable(connection, "oxcalendar_subscribed_folders",
			subscribedFoldersProperties);

	// dropTable(connection, "oxcalendar_folder");
	checkTable(connection, "oxcalendar_folder", folderProperties);

	// dropTable(connection, "oxcalendar_participants_mapping");
	checkTable(connection, "oxcalendar_participants_mapping",
			participantsMapping);

	// dropTable(connection, "ox_session");
	checkTable(connection, "ox_session", sessionStorage);

	// calendar tables
	// dropTable(connection, "oxcalendar_app_id_mapping");
	checkTable(connection, "oxcalendar_app_id_mapping",
			calendarIdMappingProperties);
	checkTable(connection, "oxcalendar_app_data", appointmentData);

	// contact tables
	// dropTable(connection, "oxcontact_id_mapping");
	checkTable(connection, "oxcontact_id_mapping", contactIdMappingProperties);
	checkTable(connection, "oxcontact_addressbook_mapping", addressbookMapping);
}

function checkOldTables(connection) {
	FW_LOG("check for old tables");
	// check if old table structure exists
	// 'oxcalendar_app_id_mapping' should have column storage_id anymore
	if (connection.tableExists("oxcalendar_app_id_mapping")) {
		var SQL = "SELECT folder FROM oxcalendar_app_id_mapping LIMIT 1";
		try {
			connection.executeSimpleSQL(SQL);
		} catch (e) {
			FW_LOG("old table detected, drop table and create new structure");
			dropTable(connection, "oxcalendar_app_id_mapping");
		}
	}
}

function dropTable(connection, tableName) {
	var SQL = "DROP TABLE " + tableName;
	if (connection.tableExists(tableName)) {
		FW_LOG("TABLE " + tableName + " exists, try to drop it");
		FW_LOG("SQL: " + SQL);
		connection.executeSimpleSQL(SQL);
	}
}

function checkTable(connection, tableName, tableProperties) {
	if (!connection.tableExists(tableName)) {
		FW_LOG("TABLE " + tableName + " doesn't exist, try to create it");

		var fieldCounter = 0;
		var CREATE_SQL = "CREATE TABLE " + tableName + " (";
		for ( var fieldName in tableProperties) {
			var fieldType = tableProperties[fieldName];

			if (fieldCounter == 0) {
				CREATE_SQL += fieldName + " " + fieldType;
			} else {
				CREATE_SQL += ", " + fieldName + " " + fieldType;
			}

			fieldCounter++;
		}
		CREATE_SQL += ")";

		FW_LOG("EXECUTE SQL: " + CREATE_SQL);
		connection.executeSimpleSQL(CREATE_SQL);
		FW_LOG("TABLE " + tableName + " CREATED");
	} else {
		FW_LOG("TABLE " + tableName + " already exists, do nothing");
	}
}

// ######################################
// #### PREFERENCES METHODS #############
// ######################################

function getOXCalendarPref(connection, name) {
	var SQL = "SELECT value FROM oxcalendar_preferences WHERE name = ?1";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, name);

	var retval;

	if (statement.executeStep()) {
		retval = statement.getString(0);
	}

	statement.reset();
	statement.finalize();

	return retval;
}

function saveOXCalendarPref(connection, name, value) {
	FW_LOG("saveOXCalendarPref: name=" + name);
	var statement;
	var oldValue = getOXCalendarPref(connection, name);
	if (oldValue != undefined) {
		FW_LOG("old value found in preference " + name + ", trying to update");
		statement = connection
				.createStatement("UPDATE oxcalendar_preferences SET value = ?1 WHERE name = ?2");
		statement.bindUTF8StringParameter(0, value);
		statement.bindUTF8StringParameter(1, name);
		FW_LOG("update finished");
	} else {
		FW_LOG("no preference " + name + " found, create a new one");
		statement = connection
				.createStatement("INSERT INTO oxcalendar_preferences (name, value) VALUES (?1, ?2)");
		statement.bindUTF8StringParameter(0, name);
		statement.bindUTF8StringParameter(1, value);
		FW_LOG("preference created");
	}

	statement.execute();
	FW_LOG("saveOXCalendarPref statement executed");

	statement.reset();
	FW_LOG("reset saveOXCalendarPref statement");

	statement.finalize();
}

function deleteOXCalendarPref(connection, name) {
	FW_LOG("deleteOXCalendarPref connection=" + connection + ", name=" + name);
	var SQL = "DELETE FROM oxcalendar_preferences WHERE name = ?";

	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, name);

	statement.execute();
	statement.reset();

	statement.finalize();

}

// ######################################
// ##### ID MAPPING METHODS #############
// ######################################

function getContactSyncObjects(connection, addressbookId) {
	FW_LOG("getSyncObjects addressbookId=" + addressbookId);
	var SQL = "SELECT local_id, remote_id, local_timestamp, remote_folder, remote_timestamp FROM oxcontact_id_mapping WHERE storage_id = ?1";
	FW_LOG("getContactSyncObjects: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, addressbookId);

	var contactSyncObjects = getCommonSyncObjectsByStatement(statement);
	return contactSyncObjects;
}

function getCalendarSyncObjects(connection, folder) {
	FW_LOG("getSyncObjects for folder: " + folder);
	var SQL = "SELECT local_id, remote_id, local_timestamp, remote_folder, remote_timestamp FROM oxcalendar_app_id_mapping WHERE folder = ?1";
	FW_LOG("getSyncObjects: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);

	var calendarSyncObjects = getCommonSyncObjectsByStatement(statement);
	return calendarSyncObjects;
}

function getCommonSyncObjectsByStatement(statement) {
	var retval = [];

	while (statement.executeStep()) {
		var localId = statement.getString(0);
		var remoteId = statement.getString(1);
		var localTimestamp = statement.getString(2);
		var remoteFolder = statement.getString(3);
		var remoteTimestamp = statement.getString(4);

		var syncObj = {};
		syncObj['local_id'] = localId;
		syncObj['local_timestamp'] = localTimestamp;
		syncObj['remote_id'] = remoteId;
		syncObj['remote_folder'] = remoteFolder;
		syncObj['remote_timestamp'] = remoteTimestamp;

		retval.push(syncObj);
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function getContactSyncObjectByRemoteId(connection, addressbookId, remoteId) {
	FW_LOG("getContactSyncObjectByRemoteId addressbookId=" + addressbookId
			+ ", remoteId=" + remoteId);
	var SQL = "SELECT local_id, local_timestamp, remote_folder, remote_timestamp FROM oxcontact_id_mapping WHERE storage_id = ?1 AND remote_id = ?2";
	FW_LOG("getContactSyncObjectByRemoteId: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, addressbookId);
	statement.bindUTF8StringParameter(1, remoteId);

	var syncObj = getCommonSyncObjectByRemoteId(statement, remoteId);
	return syncObj;
}

function getCalendarSyncObjectByRemoteId(connection, folder, remoteId) {
	FW_LOG("getCalendarSyncObjectByRemoteId storageId=" + folder
			+ ", remoteId=" + remoteId);
	var SQL = "SELECT local_id, local_timestamp, remote_folder, remote_timestamp FROM oxcalendar_app_id_mapping WHERE folder = ?1 AND remote_id = ?2";
	FW_LOG("getCalendarSyncObjectByRemoteId: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);
	statement.bindUTF8StringParameter(1, remoteId);

	var syncObj = getCommonSyncObjectByRemoteId(statement, remoteId);
	return syncObj;
}

function getCommonSyncObjectByRemoteId(statement, remoteId) {
	var retval;

	if (statement.executeStep()) {
		var localId = statement.getString(0);
		retval = _syncStatement2SyncObject(statement, localId, remoteId);

	}

	statement.reset();

	statement.finalize();

	return retval;
}

function getContactSyncObjectByLocalId(connection, addressbookId, localId) {
	var SQL = "SELECT remote_id, local_timestamp, remote_folder, remote_timestamp FROM oxcontact_id_mapping WHERE storage_id = ?1 AND local_id = ?2";
	FW_LOG("getContactSyncObjectByLocalId SQL: " + SQL + " storageId="
			+ addressbookId + ",localId=" + localId);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, addressbookId);
	statement.bindUTF8StringParameter(1, localId);

	var syncObj = getCommonSyncObjectByLocalId(statement);
	return syncObj;
}

function getCalendarSyncObjectByLocalId(connection, folder, localId) {
	var SQL = "SELECT remote_id, local_timestamp, remote_folder, remote_timestamp FROM oxcalendar_app_id_mapping WHERE folder = ?1 AND local_id = ?2";
	FW_LOG("getCalendarSyncObjectByLocalId SQL: " + SQL + " folder=" + folder
			+ ",localId=" + localId);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);
	statement.bindUTF8StringParameter(1, localId);

	var syncObj = getCommonSyncObjectByLocalId(statement, localId);
	return syncObj;
}

function getCommonSyncObjectByLocalId(statement, localId) {
	var retval;

	if (statement.executeStep()) {
		var remoteId = statement.getString(0);
		retval = _syncStatement2SyncObject(statement, localId, remoteId);
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function _syncStatement2SyncObject(statement, localId, remoteId) {
	var retval = {};
	var localTimestamp = statement.getString(1);

	var remoteFolder = statement.getString(2);
	var remoteTimestamp = statement.getString(3);

	retval['local_id'] = localId;
	retval['local_timestamp'] = localTimestamp;
	retval['remote_id'] = remoteId;
	retval['remote_folder'] = remoteFolder;
	retval['remote_timestamp'] = remoteTimestamp;

	return retval;
}

function getRemoteIdArray(connection, storageId, tableName) {
	return _getIdArrayByAttribute(connection, storageId, 'remote_id', tableName);
}

function getLocalIdArray(connection, storageId, tableName) {
	return _getIdArrayByAttribute(connection, storageId, 'local_id', tableName);
}

function _getIdArrayByAttribute(connection, storageId, attribute, tableName) {
	var SQL = "SELECT " + attribute + " FROM " + tableName
			+ " WHERE storage_id = ?1";
	FW_LOG("_getIdArrayByAttribute: SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, storageId);

	var retval = [];

	while (statement.executeStep()) {
		retval[retval.length] = statement.getString(0);
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function insertContactIdMapping(connection, addressbookId, localId,
		localTimestamp, remoteId, remoteFolder, remoteTimestamp) {
	FW_LOG("insertContactIdMapping: connection=" + connection
			+ ", addressbookId=" + addressbookId + ", localId=" + localId
			+ ", localTimestamp=" + localTimestamp + ", remoteId=" + remoteId
			+ ", remoteFolder=" + remoteFolder + ", remoteTimestamp="
			+ remoteTimestamp);

	var SQL = "INSERT INTO oxcontact_id_mapping (storage_id, local_id, local_timestamp, remote_id, remote_folder, remote_timestamp) VALUES (?1, ?2, ?3, ?4, ?5, ?6)";
	FW_LOG("insertContactIdMapping SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, addressbookId);

	insertCommonIdMapping(statement, localId, localTimestamp, remoteId,
			remoteFolder, remoteTimestamp);
}

function insertCalendarIdMapping(connection, folder, localId, localTimestamp,
		remoteId, remoteFolder, remoteTimestamp) {
	FW_LOG("insertIdMappingEntry: connection=" + connection + ", folder="
			+ folder + ", localId=" + localId + ", localTimestamp="
			+ localTimestamp + ", remoteId=" + remoteId + ", remoteFolder="
			+ remoteFolder + ", remoteTimestamp=" + remoteTimestamp);

	var SQL = "INSERT INTO oxcalendar_app_id_mapping (folder, local_id, local_timestamp, remote_id, remote_folder, remote_timestamp) VALUES (?1, ?2, ?3, ?4, ?5, ?6)";
	FW_LOG("insertIdCalendarMapping SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);

	insertCommonIdMapping(statement, localId, localTimestamp, remoteId,
			remoteFolder, remoteTimestamp);
}

function insertCommonIdMapping(statement, localId, localTimestamp, remoteId,
		remoteFolder, remoteTimestamp) {
	FW_LOG("insertCommonIdMapping: localId=" + localId + ", localTimestamp="
			+ localTimestamp + ", remoteId=" + remoteId + ", remoteFolder="
			+ remoteFolder + ", remoteTimestamp=" + remoteTimestamp);

	statement.bindUTF8StringParameter(1, String(localId));
	statement.bindUTF8StringParameter(2, String(localTimestamp));
	statement.bindUTF8StringParameter(3, String(remoteId));
	statement.bindUTF8StringParameter(4, String(remoteFolder));
	statement.bindUTF8StringParameter(5, String(remoteTimestamp));

	statement.execute();
	statement.reset();

	statement.finalize();

}

function updateContactMappingTimestamp(connection, addressbookId, localId,
		fieldName, timestamp) {
	FW_LOG("updateTimestamp addressbookId=" + addressbookId + ", localId="
			+ localId + ", fieldName=" + fieldName + ", timestamp=" + timestamp);
	var SQL = "UPDATE oxcontact_id_mapping SET " + fieldName
			+ " = ?1 WHERE storage_id = ?2 AND local_id = ?3";
	var statement = connection.createStatement(SQL);
	updateCommonMappingTimestamp(statement, addressbookId, localId, timestamp);
}

function updateCalendarMappingTimestamp(connection, folder, localId, fieldName,
		timestamp) {
	FW_LOG("updateCalendarMappingTimestamp folder=" + folder + ", localId="
			+ localId + ", fieldName=" + fieldName + ", timestamp=" + timestamp);
	var SQL = "UPDATE oxcalendar_app_id_mapping SET " + fieldName
			+ " = ?1 WHERE storage_id = ?2 AND local_id = ?3";
	var statement = connection.createStatement(SQL);
	updateCommonMappingTimestamp(statement, folder, localId, timestamp);
}

function updateCommonMappingTimestamp(statement, storageId, localId, timestamp) {
	statement.bindUTF8StringParameter(0, String(timestamp));
	statement.bindInt32Parameter(1, storageId);
	statement.bindUTF8StringParameter(2, String(localId));

	statement.execute();
	statement.reset();

	statement.finalize();

}

function deleteContactIdMapping(connection, addressbookId, localIdArray) {
	FW_LOG("deleteContactIdMapping connection=" + connection
			+ ", addressbookId=" + addressbookId + " deleting entries: "
			+ localIdArray.toSource());
	var SQL = "DELETE FROM oxcontact_id_mapping WHERE storage_id = ?1 AND local_id IN ("
			+ buildCommonSQLDeletePart(localIdArray) + ")";

	FW_LOG("deleteContactIdMappingEntry: SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, addressbookId);

	deleteCommonIdMapping(statement, localIdArray);
}

function deleteCalendarIdMapping(connection, folder, localIdArray) {
	FW_LOG("deleteCalendarIdMapping connection=" + connection + ", folder="
			+ folder + ", deleting entries: " + localIdArray.toSource());
	var SQL = "DELETE FROM oxcalendar_app_id_mapping WHERE folder = ?1 AND local_id IN ("
			+ buildCommonSQLDeletePart(localIdArray) + ")";

	FW_LOG("deleteAppointmentIdMapping: SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);

	deleteCommonIdMapping(statement, localIdArray);
}

function buildCommonSQLDeletePart(localIdArray) {
	var sqlDeletePart = "";

	for ( var a = 0; a < localIdArray.length; a++) {
		if (a == 0) {
			sqlDeletePart += "?" + (a + 2);
		} else {
			sqlDeletePart += ",?" + (a + 2);
		}
	}

	return sqlDeletePart;
}

function deleteCommonIdMapping(statement, localIdArray) {
	for ( var a = 0; a < localIdArray.length; a++) {
		var localId = localIdArray[a];

		statement.bindUTF8StringParameter((a + 1), localId);
	}

	statement.execute();
	statement.reset();

	statement.finalize();

}

function saveAppointmentData(connection, folderId, localId, start, end,
		appointmentData) {
	var exists = existsAppointmentDataInFolder(connection, folderId, localId);
	FW_LOG("saveAppointmentData: folderId=" + folderId + ", localId=" + localId
			+ ", exists=" + exists);

	if (exists) {
		updateAppointmentData(connection, folderId, localId, start, end,
				appointmentData);
	} else {
		insertAppointmentData(connection, folderId, localId, start, end,
				appointmentData);
	}
}

function insertAppointmentData(connection, folderId, localId, start, end,
		appointmentData) {
	var statement = connection
			.createStatement("INSERT INTO oxcalendar_app_data (folder, local_id, start, end, data) VALUES (?1, ?2, ?3, ?4, ?5)");
	statement.bindInt32Parameter(0, folderId);
	statement.bindUTF8StringParameter(1, localId);
	statement.bindInt64Parameter(2, start);
	statement.bindInt64Parameter(3, end);
	statement.bindUTF8StringParameter(4, appointmentData);

	statement.execute();
	statement.reset();

	statement.finalize();

}

function updateAppointmentData(connection, folderId, localId, start, end,
		appointmentData) {
	var SQL = "UPDATE oxcalendar_app_data SET start = ?1, end = ?2, data = ?3 WHERE folder = ?4 AND local_id = ?5";
	var statement = connection.createStatement(SQL);
	statement.bindInt64Parameter(0, start);
	statement.bindInt64Parameter(1, end);
	statement.bindUTF8StringParameter(2, appointmentData);
	statement.bindInt32Parameter(3, folderId);
	statement.bindUTF8StringParameter(4, localId);

	statement.execute();
	statement.reset();

	statement.finalize();

}

function deleteAppointmentsData(connection, folderId, localIdArray) {
	for ( var a = 0; a < localIdArray.length; a++) {
		var localId = localIdArray[a];
		var SQL = "DELETE FROM oxcalendar_app_data WHERE folder = ?1 AND local_id = ?2";
		var statement = connection.createStatement(SQL);
		statement.bindInt32Parameter(0, folderId);
		statement.bindUTF8StringParameter(1, localId);

		statement.execute();
		statement.reset();

		statement.finalize();

	}
}

function listAppointmentsInRange(connection, folder, startAsLong, endAsLong) {
	FW_LOG("listAppointmentsInRange folder=" + folder + ", startAsLong="
			+ startAsLong + ", endAsLong=" + endAsLong);

	var SQL = "SELECT local_id, data FROM oxcalendar_app_data WHERE folder = ?1 AND start <= ?2 AND end >= ?3";
	FW_LOG("SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);

	// range query --> start >= rangeEnd and end <= rangeStart
	statement.bindInt64Parameter(1, endAsLong);
	statement.bindInt64Parameter(2, startAsLong);

	var appointments = [];
	while (statement.executeStep()) {
		var localId = statement.getString(0);
		var appointmentAsString = statement.getString(1);
		// FW_LOG("appointmentAsString: " + appointmentAsString);

		// var appointment = Function("return " + appointmentAsString);
		// //FXJSON.fromString(appointmentAsString);
		var appointment = FXJSON.fromString(appointmentAsString);
		appointment['local_id'] = localId;

		// FW_LOG("appointment: " + appointment.toSource());
		appointments.push(appointment);
	}

	statement.reset();

	statement.finalize();

	return appointments;
}

function searchAppointmentsByTitle(connection, title, startAsLong, endAsLong) {
	FW_LOG("searchAppointments: title=" + title + ", startAsLong="
			+ startAsLong + ", endAsLong=" + endAsLong);

	var SQL = "SELECT local_id, data FROM oxcalendar_app_data WHERE start <= ?1 AND end >= ?2";
	FW_LOG("SQL: " + SQL);
	var statement = connection.createStatement(SQL);

	// range query --> start >= rangeEnd and end <= rangeStart
	statement.bindInt64Parameter(0, endAsLong);
	statement.bindInt64Parameter(1, startAsLong);

	var appointments = [];
	while (statement.executeStep()) {
		var localId = statement.getString(0);
		var appointmentAsString = statement.getString(1);
		// FW_LOG("appointmentAsString: " + appointmentAsString);

		// var appointment = Function("return " + appointmentAsString);
		// //FXJSON.fromString(appointmentAsString);
		var appointment = FXJSON.fromString(appointmentAsString);
		appointment['local_id'] = localId;

		// FW_LOG("appointment: " + appointment.toSource());
		if (appointment['title'] == title) {
			appointments.push(appointment);
		}
	}

	statement.reset();

	statement.finalize();

	return appointments;
}

function loadAppointmentData(connection, folder, localId) {
	var appointment;

	FW_LOG("loadAppointmentData folder=" + folder + ", localId=" + localId);

	var SQL = "SELECT data FROM oxcalendar_app_data WHERE folder = ?1 AND local_id = ?2";
	FW_LOG("SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);
	statement.bindUTF8StringParameter(1, localId);

	if (statement.executeStep()) {
		var appointmentAsString = statement.getString(0);
		appointment = FXJSON.fromString(appointmentAsString);
		appointment['local_id'] = localId;
	}

	statement.reset();

	statement.finalize();

	return appointment;
}

function deleteAppointmentData(connection, folderId, localId) {
	FW_LOG("deleteAppointmentData: folderId=" + folderId + ", localId="
			+ localId);
	var SQL = "DELETE FROM oxcalendar_app_data WHERE folder = ?1 AND local_id = ?2";
	FW_LOG("SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folderId);
	statement.bindUTF8StringParameter(1, localId);

	statement.execute();
	statement.reset();

	statement.finalize();

}

function listAppLocalIdsForFolder(connection, folder) {
	var SQL = "SELECT local_id FROM oxcalendar_app_data WHERE folder = ?1";
	FW_LOG("listAppointmentIdsForFolder: SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);

	var retval = [];

	while (statement.executeStep()) {
		var localId = statement.getString(0);
		retval.push(localId);
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function existsAppointmentId(connection, localId) {
	FW_LOG("existAppointmentId: localId=" + localId);
	var SQL = "SELECT folder FROM oxcalendar_app_data WHERE local_id = ?1";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, localId);

	var folder = -1;
	if (statement.executeStep()) {
		folder = statement.getInt32(0);
	}

	statement.reset();

	statement.finalize();

	return folder;
}

function existsAppointmentDataInFolder(connection, folder, localId) {
	FW_LOG("existsAppointmentDataInFolder: folder=" + folder + ", localId="
			+ localId);
	var SQL = "SELECT folder FROM oxcalendar_app_data WHERE folder = ?1 AND local_id = ?2";
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);
	statement.bindUTF8StringParameter(1, localId);

	var exists = false;
	if (statement.executeStep()) {
		exists = true;
	}

	statement.reset();

	statement.finalize();

	return exists;
}

function loadAppointmentDataById(connection, folder, localId) {
	var SQL = "SELECT data FROM oxcalendar_app_data WHERE folder = ?1 AND local_id = ?2";
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, folder);
	statement.bindUTF8StringParameter(1, localId);

	var retval;

	if (statement.executeStep()) {
		var appointmentObjAsString = statement.getString(0);

		retval = FXJSON.fromString(appointmentObjAsString);
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function getNewestContactLastModified(connection, addressbookId) {
	FW_LOG("getNewestContactLastModified addressbookId=" + addressbookId);
	var SQL = "SELECT remote_timestamp FROM oxcontact_id_mapping WHERE storage_id = ? ORDER BY CAST(remote_timestamp as INT) DESC LIMIT 1";
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, addressbookId);

	var lastModified = 0;

	if (statement.executeStep()) {
		lastModified = Number(statement.getString(0)) - (12 * 60 * 60 * 1000);
	}

	statement.reset();

	statement.finalize();

	return lastModified;
}

// ######################################
// #### SUBSCRIBED FOLDERS METHODS ######
// ######################################

function loadSubscribedFolders(connection, calendarId) {
	var SQL = "SELECT id, module FROM oxcalendar_subscribed_folders WHERE calendar_id = ?1";
	FW_LOG("loadSubscribedFolders: SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, calendarId);

	var retval = [];

	while (statement.executeStep()) {
		var id = statement.getString(0);
		var module = statement.getString(0);

		retval[retval.length] = {
			'id' : id,
			'module' : module
		};
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function loadSubscribedFolder(connection, calendarId, id) {
	var SQL = "SELECT id, module FROM oxcalendar_subscribed_folders WHERE calendar_id = ?1 AND id = ?2";
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, calendarId);
	statement.bindUTF8StringParameter(1, id);

	var retval;

	if (statement.executeStep()) {
		var folderId = statement.getString(0);
		var module = statement.getString(1);

		retval = {
			'id' : folderId,
			'module' : module
		};
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function saveSubscribedFolder(connection, calendarId, id, module) {
	FW_LOG("saveSubscribedFolder: calendarId=" + calendarId + ", id=" + id);
	var statement;
	var oldValue = loadSubscribedFolder(connection, calendarId, id);
	if (!oldValue) {
		FW_LOG("subscribed folder not found, create a new mapping");
		statement = connection
				.createStatement("INSERT INTO oxcalendar_subscribed_folders (calendar_id, id, module) VALUES (?1, ?2, ?3)");
		statement.bindInt32Parameter(0, calendarId);
		statement.bindUTF8StringParameter(1, id);
		statement.bindUTF8StringParameter(2, module);
		FW_LOG("subscribed folder entry created");

		statement.execute();
		FW_LOG("saveSubscribedFolder statement executed");

		statement.reset();

		statement.finalize();

		FW_LOG("reset saveSubscribedFolder statement");
	}
}

// ######################################
// ### PARTICIPANTS MAPPING METHODS #####
// ######################################

function getAllParticipantDetails(connection) {
	FW_LOG("getAllParticipantDetails");
	var SQL = "SELECT id, type, display_name, mail FROM oxcalendar_participants_mapping";
	FW_LOG("getAllParticipantDetails: SQL=" + SQL);
	var statement = connection.createStatement(SQL);

	var retval = {};

	while (statement.executeStep()) {
		var id = statement.getString(0);
		if (!id) {
			continue;
		}

		var type = statement.getInt32(1);
		var display_name = statement.getString(2);
		var mail = statement.getString(3);

		retval[id] = {
			'display_name' : display_name,
			'mail' : mail,
			'type' : type
		};
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function getParticipantDetails(connection, calendarId, id, type) {
	FW_LOG("getParticipantsDetails: calendarId=" + calendarId + " id=" + id
			+ ", type=" + type);
	var SQL = "SELECT display_name, mail FROM oxcalendar_participants_mapping WHERE calendar_id = ?1 AND id = ?2 AND type = ?3";
	FW_LOG("getParticipantsDetails: SQL=" + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, calendarId);
	statement.bindUTF8StringParameter(1, id);
	statement.bindInt32Parameter(2, type);

	var retval;

	if (statement.executeStep()) {
		var display_name = statement.getString(0);
		var mail = statement.getString(1);

		FW_LOG("participant resolved to: " + display_name);
		retval = {
			'display_name' : display_name,
			'mail' : mail
		};
	} else {
		FW_LOG("participant id not found in local storage");
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function getParticipantIdByAttribute(connection, calendarId, attributeName,
		attributeValue, type) {
	FW_LOG("getParticipantByAttribute calendarId=" + calendarId
			+ ", attributeName=" + attributeName + ", attributeValue="
			+ attributeValue + ", type=" + type);
	var SQL = "SELECT id FROM oxcalendar_participants_mapping WHERE calendar_id = ?1 AND "
			+ attributeName + " = ?2 AND type = ?3";
	FW_LOG("getParticipantByAttribute SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, calendarId);
	statement.bindUTF8StringParameter(1, attributeValue);
	statement.bindInt32Parameter(2, type);

	var retval;

	if (statement.executeStep()) {
		retval = statement.getString(0)
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function saveParticipantMapping(connection, calendarId, id, display_name, mail,
		type) {
	FW_LOG("saveParticipantMapping: calendarId=" + calendarId + ", id=" + id);
	var oldValue = getParticipantDetails(connection, calendarId, id, type);
	if (!oldValue) {
		insertParticipantMapping(connection, calendarId, id, display_name,
				mail, type);
	}
}

function insertParticipantMapping(connection, calendarId, id, display_name,
		mail, type) {
	FW_LOG("no participant mapping found, create a new mapping");
	var statement = connection
			.createStatement("INSERT INTO oxcalendar_participants_mapping (calendar_id, id, display_name, mail, type) VALUES (?1, ?2, ?3, ?4, ?5)");
	statement.bindInt32Parameter(0, calendarId);
	statement.bindUTF8StringParameter(1, id);
	statement.bindUTF8StringParameter(2, display_name);
	statement.bindUTF8StringParameter(3, mail);
	statement.bindInt32Parameter(4, type);
	FW_LOG("participants mapping created");

	statement.execute();
	statement.reset();

	statement.finalize();

}

function updateParticipantMapping(connection, calendarId, id, display_name,
		mail, type) {
	FW_LOG("participant mapping already exists, try to update");
	var SQL = "UPDATE oxcalendar_participants_mapping SET display_name = ?1, mail = ?2 WHERE calendar_id = ?3 AND id = ?4 AND type = ?5";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, display_name);
	statement.bindUTF8StringParameter(1, mail);
	statement.bindInt32Parameter(2, calendarId);
	statement.bindUTF8StringParameter(3, id);
	statement.bindInt32Parameter(4, type);
	FW_LOG("update participant mapping");

	statement.execute();
	statement.reset();

	statement.finalize();

}

function deleteParticipantMapping(connection, calendarId, id, type) {
	FW_LOG("deleteParticipantMapping: calendarId=" + calendarId + ", id=" + id
			+ ", type=" + type);
	var SQL = "DELETE FROM oxcalendar_participants_mapping WHERE calendar_id = ?1 AND id = ?2 AND type = ?3";
	// FW_LOG("SQL: "+ SQL);
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, calendarId);
	statement.bindUTF8StringParameter(1, id);
	statement.bindInt32Parameter(2, type);
	statement.execute();
	statement.reset();

	statement.finalize();

}

// ######################################
// ### FOLDER MAPPING METHODS ###########
// ######################################

function saveFolderDetails(connection, folderObj) {
	FW_LOG("saveFolderDetails: folderObj=" + folderObj.toSource());
	var oldValue = getFolderDetails(connection, folderObj.id, folderObj.module);
	if (!oldValue) {
		insertFolderDetails(connection, folderObj);
	}
}

function insertFolderDetails(connection, folderObj) {
	FW_LOG("no folder mapping found, create a new mapping");
	var statement = connection
			.createStatement("INSERT INTO oxcalendar_folder (id, created_by, name, module, type, timestamp) VALUES (?1, ?2, ?3, ?4, ?5, ?6)");
	statement.bindUTF8StringParameter(0, String(folderObj.id));
	statement.bindUTF8StringParameter(1, folderObj.created_by);
	statement.bindUTF8StringParameter(2, folderObj.name);
	statement.bindUTF8StringParameter(3, folderObj.module);
	statement.bindUTF8StringParameter(4, folderObj.type);
	statement.bindUTF8StringParameter(5, String(folderObj.last_modified));
	FW_LOG("folder details inserted");

	statement.execute();
	statement.reset();

	statement.finalize();

}

function updateFolderDetails(connection, folderObj) {
	FW_LOG("no folder mapping found, create a new mapping");
	var SQL = "UPDATE oxcalendar_folder SET created_by = ?1, name = ?2, timestamp = ?3 WHERE id = ?4";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, folderObj.created_by);
	statement.bindUTF8StringParameter(1, folderObj.name);
	statement.bindUTF8StringParameter(2, String(folderObj.last_modified));
	statement.bindUTF8StringParameter(3, String(folderObj.id));
	FW_LOG("folder details updated");

	statement.execute();
	statement.reset();

	statement.finalize();

}

function deleteFolderMapping(connection, id) {
	FW_LOG("deleteParticipantMapping: id=" + id);
	var SQL = "DELETE FROM oxcalendar_folder WHERE id = ?1";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, id);
	statement.execute();
	statement.reset();

	statement.finalize();

}

/**
 * @deprecated
 * @param connection
 * @returns {___anonymous40041_40043}
 */
function getAllFolderDetails(connection) {
	FW_LOG("getAllFolderDetails");
	var SQL = "SELECT id, created_by, name, type, module, timestamp FROM oxcalendar_folder";
	var statement = connection.createStatement(SQL);

	var retval = {};

	while (statement.executeStep()) {
		var id = statement.getString(0);
		var created_by = statement.getString(1);
		var name = statement.getString(2);
		var type = statement.getString(3);
		var module = statement.getString(4);
		var timestamp = statement.getString(5);

		retval[id] = {
			'id' : id,
			'created_by' : created_by,
			'name' : name,
			'type' : type,
			'module' : module,
			'timestamp' : timestamp
		};
	}

	statement.reset();

	statement.finalize();

	return retval;
}
function getAllFolderDetailsByModule(connection, module) {
	FW_LOG("getAllFolderDetailsByModule: module=" + module);

	var SQL = "SELECT id, created_by, name, type, timestamp FROM oxcalendar_folder WHERE module = ?1";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, module);

	var retval = {};

	while (statement.executeStep()) {
		var id = statement.getString(0);
		var created_by = statement.getString(1);
		var name = statement.getString(2);
		var type = statement.getString(3);
		var timestamp = statement.getString(4);
		FW_LOG("getAllFolderDetails: " + name);
		retval[id] = {
			'id' : id,
			'created_by' : created_by,
			'name' : name,
			'type' : type,
			'timestamp' : timestamp
		};
	}

	statement.reset();
	statement.finalize();

	return retval;
}

function getFolderDetails(connection, id, module) {
	FW_LOG("getFolderDetails: id=" + id + ", module=" + module);
	var SQL = "SELECT created_by, name, type, timestamp FROM oxcalendar_folder WHERE id = ?1 AND module = ?2";
	FW_LOG("getFolderDetails: SQL=" + SQL);
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, String(id));
	statement.bindUTF8StringParameter(1, module);

	var retval;

	if (statement.executeStep()) {
		var created_by = statement.getString(0);
		var name = statement.getString(1);
		var type = statement.getString(2);
		var timestamp = statement.getString(3);

		retval = {
			'created_by' : created_by,
			'name' : name,
			'type' : type,
			'timestamp' : timestamp
		};
	}

	statement.reset();

	statement.finalize();

	return retval;
}

// ######################################
// ### ADDRESSBOOK MAPPING METHODS ######
// ######################################

function saveAddressbookURI(connection, addressbookURI) {
	FW_LOG("saveAddressbookURI: addressbookURI=" + addressbookURI);
	var statement;
	var addressbookId = getAddressbookIdByURI(connection, addressbookURI);
	if (!addressbookId) {
		FW_LOG("no mapping for addressbook uri found, create a new mapping");
		statement = connection
				.createStatement("INSERT INTO oxcontact_addressbook_mapping (addressbook_uri) VALUES (?1)");
		statement.bindUTF8StringParameter(0, addressbookURI);
		FW_LOG("addressbook mapping created");

		statement.execute();
		FW_LOG("saveAddressbookURI statement executed");

		statement.reset();
		statement.finalize();

		FW_LOG("reset saveAddressbookURI statement");
	}
}

function getAddressbookIdByURI(connection, addressbookURI) {
	var addressbook = _getAddressbookByAttribute(connection, 'addressbook_uri',
			addressbookURI);
	if (addressbook) {
		return addressbook['addressbook_id'];
	}
	return undefined;
}

function getAddressbookURIById(connection, addressbookId) {
	var addressbook = _getAddressbookByAttribute(connection, 'addressbook_id',
			addressbookId);
	if (addressbook) {
		return addressbook['addressbook_uri'];
	}
	return undefined;
}

function _getAddressbookByAttribute(connection, attribute, value) {
	var SQL = "SELECT addressbook_id, addressbook_uri FROM oxcontact_addressbook_mapping WHERE "
			+ attribute + " = ?1";
	FW_LOG("_getAddressbookByAttribute SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	if (attribute == 'addressbook_id') {
		statement.bindInt32Parameter(0, value);
	} else {
		statement.bindUTF8StringParameter(0, value);
	}

	var retval;

	if (statement.executeStep()) {
		retval = {};
		retval['addressbook_id'] = statement.getInt32(0);
		retval['addressbook_uri'] = statement.getString(1);
	}

	statement.reset();

	statement.finalize();

	return retval;
}

function saveSession(connection, session, random) {
	FW_LOG("saveSession: connection=" + connection + ", session=" + session
			+ ", random=" + random);
	var statement = connection
			.createStatement("INSERT INTO ox_session (session, random) VALUES (?1, ?2)");
	statement.bindUTF8StringParameter(0, session);
	statement.bindUTF8StringParameter(1, random);

	statement.execute();
	statement.reset();

	statement.finalize();

}

function deleteSession(connection, id) {
	FW_LOG("deleteSession: id=" + id);
	var SQL = "DELETE FROM ox_session WHERE id = ?1";
	var statement = connection.createStatement(SQL);
	statement.bindInt32Parameter(0, id);
	statement.execute();
	statement.reset();

	statement.finalize();

}

function deleteSessionBySession(connection, session) {
	FW_LOG("deleteSession: session=" + session);
	var SQL = "DELETE FROM ox_session WHERE session = ?1";
	var statement = connection.createStatement(SQL);
	statement.bindUTF8StringParameter(0, session);
	statement.execute();
	statement.reset();

	statement.finalize();

}

function getNewestSession(connection) {
	FW_LOG("getNewestSession");
	var SQL = "SELECT session, random FROM ox_session ORDER BY id DESC LIMIT 1";
	FW_LOG("getNewestSession SQL: " + SQL);
	var statement = connection.createStatement(SQL);

	var session;
	var cookie;

	if (statement.executeStep()) {
		session = statement.getString(0);
		cookie = statement.getString(1);
	}

    

	statement.reset();

	statement.finalize();

	return [ session, cookie ];
}

function getOldSessions(connection) {
	FW_LOG("getOldSessions");
	var SQL = "SELECT id, session, random FROM ox_session ORDER BY id DESC";
	FW_LOG("getOldSessions SQL: " + SQL);
	var statement = connection.createStatement(SQL);
	var retval;

	var oldSessions = [];

	while (statement.executeStep()) {
		var sessionObj = {};
		sessionObj['id'] = statement.getInt32(0);
		sessionObj['session'] = statement.getString(1);
		sessionObj['random'] = statement.getString(2);

		oldSessions[oldSessions.length] = sessionObj;
	}

	statement.reset();

	statement.finalize();

	if (oldSessions.length > 5) {
		return oldSessions.slice(5);
	}

	return [];
}

// ACCESS SYSTEM TABLES
function generateNewCalendarId() {
	var kStorageServiceContractID = "@mozilla.org/storage/service;1";
	var kStorageServiceIID = Components.interfaces.mozIStorageService;

	var dbService = Components.classes[kStorageServiceContractID]
			.getService(kStorageServiceIID);
	var connection = dbService.openSpecialDatabase("profile");

	// if nothing found return 1
	var retval = 1;

	var statement = connection
			.createStatement("SELECT id FROM cal_calendars ORDER BY id DESC");
	if (statement.executeStep()) {
		retval = statement.getInt32(0) + 1;
	}

	statement.reset();

	statement.finalize();

	// connection.close();

	return retval;
}

function getFolderFromCalendar(connection, calendar) {
	var calendarId = calendar.mID;
	if (!calendarId) {
		calendarId = calendar.id;
	}

	var folder = 0;

	FW_LOG("try to read folder for calendar from preferences for calendarId "
			+ calendarId);
	var prefFolderId = getOXCalendarPref(connection, "oxcalendar." + calendarId
			+ ".folder");
	if (prefFolderId) {
		FW_LOG("found folder id for calendar '" + calendarId
				+ "' in preferences table: " + prefFolderId);
		folder = prefFolderId;
	}

	if (folder == 0) {
		FW_LOG("no calendar folder id given for calendar '" + calendar.name
				+ "' use default folder");
		folder = getOXCalendarPref(connection, "calendar_default_folder");
	}

	return folder;
}