/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

var iSyncManager = Components.interfaces.ISyncManager;
var nsISupports = Components.interfaces.nsISupports;

function SyncManager() {
    // If you only need to access your component from Javascript, uncomment the following line:
    this.wrappedJSObject = this;

    this.syncRunning = false;

    this.readyCallbacks = [ ];

    this.systemSyncEnabled = true;
    this.calendarSyncEnabled = true;
    this.addressbookSyncEnabled = true;
}

// class definition
SyncManager.prototype = {

    QueryInterface: function(aIID) {
        if (!aIID.equals(iSyncManager) && !aIID.equals(nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    },

    preInit: function() {
        FW_LOG("SyncManager.preInit");
        initContactObserver();
    },

    /*
     * param readyCallback: The callback method of the sync has finished
     * param options: Some options to customize the sync process
     *  - system_sync_enabled: boolean (enables or disables the system sync)
     *  - calendar_sync_enabled: boolean (enables or disables the calendar sync)
     *  - addressbook_sync_enabled: boolean (enables or disables the addressbook sync)
     */
    init: function(readyCallback, options) {
        FW_LOG("sync manager init");

        updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.sync-started'));

        this.systemSyncEnabled = true;
        this.calendarSyncEnabled = true;
        this.addressbookSyncEnabled = true;

        if (options) {
            if (options['system_sync_enabled']) {
                this.systemSyncEnabled = true;
            } else {
                this.systemSyncEnabled = false;
            }

            if (options['calendar_sync_enabled']) {
                this.calendarSyncEnabled = true;
            } else {
                this.calendarSyncEnabled = false;
            }

            if (options['addressbook_sync_enabled']) {
                this.addressbookSyncEnabled = true;
            } else {
                this.addressbookSyncEnabled = false;
            }
        }

        this.folders = [ ];
        this.pos = -1;

        this.connectionDB = getOXCalendarDBConnection();

        this.HOST = getOXCalendarPref(this.connectionDB, "host");
        FW_LOG("syncManager HOST: " + this.HOST);

        if (!this.HOST) {
            FW_WARN("missing open-xchange host, cannot start sync manager");
            updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.missing-host'), "font-weight:bold;color:#660000");
            return;
        }

        // get server type
        this.serverType = getOXCalendarPref(this.connectionDB, "server-type");
        if (!this.serverType) {
            FW_WARN("unknown server type detected, please check ox url: " + this.HOST);
            updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.unknown-server-type'), "font-weight:bold;color:#660000");
            return;
        }

        if (!(this.serverType == "OX5" || this.serverType == "OX6")) {
            FW_WARN("invalid server type: " + this.serverType);
            return;
        }

        var iStatusManager = Components.classes['@fusionworkx.com/common/statusmanager;1'].getService().wrappedJSObject;

        var isOnline = hasOnlineConnection(this.HOST);
        iStatusManager.setStatus("online", isOnline);

        if (!isOnline) {
            FW_LOG("no connection to server, running connector in offline mode");
            updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.offline-mode'));
            return;
        }

        this.syncRunning = true;
        this.readyCallbacks = [ ];
        if (readyCallback) {
            this.readyCallbacks.push(readyCallback);
        }

        // check ox credentials
        var username = getOXCalendarPref(this.connectionDB, "username");
        var password = getOXCalendarPref(this.connectionDB, "password");

        this.USER = username;
        this.PASSWORD = password;

        if (username != undefined && password != undefined) {
            FW_LOG("credentials in preferences found, start sync process");
            this.dispatchSync();
        } else {
            FW_LOG("no credentials found, open dialog");
            updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.missing-credentials'), "font-weight:bold;color:#660000");
        }
    },

    addReadyCallback: function(readyCallback) {
        this.readyCallbacks.push(readyCallback);
    },

    dispatchSync: function() {
        this.iCacheManager = Components.classes['@fusionworkx.com/common/cachemanager;1'].getService().wrappedJSObject;

//        var syncThread = new SyncThread(this)//
//        if (isTB3) {
//            FW_LOG("dispatch sync thread");
//
//            var threadManager = Components.classes["@mozilla.org/thread-manager;1"].getService();
//            var target = threadManager.newThread(0);
//            this.mainThread = threadManager.mainThread;
//            target.dispatch(syncThread, target.DISPATCH_NORMAL);
//          
////            FIXME: starting with Gecko 2.0 ThreadManager (--> code above) cannot be use from XPCOM, so we would need to use
////            		something like ChromeWorker: here we could load an external script ... 
////            		but currently the code to be used in the thread is not in a single file
////            		so this would have to be reorganized (or some other threading-solution has to be used)...
////            
////            		... so something along the lines of: (except we have to implement onmessage in TRHEAD_SCRIPT.js along with all other needed code ... or export needed threading-code into another XPCOM component)
////            var worker = Components.classes["@mozilla.org/threads/workerfactory;1"]
////            				.createInstance(Components.interfaces.nsIWorkerFactory).newChromeWorker("TRHEAD_SCRIPT.js");
////            worker.syncManager = this;
////            worker.onmessage = function(evt){
////                if (this.syncManager.systemSyncEnabled) {
////                    startSystemSync(this.syncManager);
////                } else {
////                    FW_LOG("system sync disabled, call systemSyncReady method")
////                    systemSyncReady(this.syncManager);
////                }
////            };
////            worker.postMessage();
//        } else {
            if (this.systemSyncEnabled) {
                startSystemSync(this);
            } else {
                FW_LOG("system sync disabled, call systemSyncReady method")
                systemSyncReady(this);
            }
//        }

    },

    startCalendarSync: function() {
        this.pos++;

        // parameters for i18n string. First parameter is the current calendar pos
        // seconds parameter is the max number of calendars
        var params = [ String(this.pos+1), String(this.folders.length) ];        
        var calendarMessage = getI18nString('oxcalendar-provider.statusbar.reading-calendars', params);

        updateStatusBarWithThread(calendarMessage);

        FW_LOG("startCalendarSync: read folder at pos " + this.pos + " from " + this.folders.length + " folders");

        if (!this.calendarSyncEnabled) {
            FW_LOG("calendar sync disabled, call calendarSyncReady method");
            calendarSyncReady(this);
            return;
        }
        
        // read this parameter from oxtender config
        var onlySubscribedFolders = false;
        var subscribedFolders = { };
        
        var calendars = getCalendarManager().getCalendars({});

        for each (var calendar in calendars) {
            if (calendar.type == "oxcalendar") {
                var isDisabled = calendar.getProperty("disabled");
                var currentStatus = calendar.getProperty("currentStatus");
                
                if (isDisabled) {
                  // if the calendar is disabled don't add it to the subscribed 
                  // folders
                  FW_LOG("calendar " + calendar.name  + " disabled");
                  continue;
                } else {
                  FW_LOG("calendar " + calendar.name  + " NOT disabled");
                }
                var folder = getFolderFromCalendar(this.connectionDB, calendar);
                subscribedFolders[folder] = true;
            }
        }

        if (this.folders.length > 0 && this.pos < this.folders.length) {
            if (onlySubscribedFolders) {
              if (subscribedFolders[this.folders[this.pos]]) {
                syncCalendar(this, this.folders[this.pos]);
              } else {
                this.startCalendarSync();
              }
            } else {
              syncCalendar(this, this.folders[this.pos]);
            }
        } else {
            calendarSyncReady(this);
        }
    },

    startAddressbookSync: function() {
        FW_LOG("startAddressbookSync");

        updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.reading-server-addressbooks'));

        if (this.addressbookSyncEnabled) {
            startNewContactSync(this);
        } else {
            FW_LOG("addressbook sync disabled, call addressbookSyncReady");
            addressbookSyncReady(this);
        }
    },

    notifyReadyCallbacks: function() {
        FW_LOG("notifyReadyCallbacks: " + this.readyCallbacks.length + " callbacks detected");
        for (var a = 0; a < this.readyCallbacks.length; a++) {
            var callback = this.readyCallbacks[a];
            callback();
        }
    }
};

/*
 * SyncThread
 */
function SyncThread(syncManager) {
    this.syncManager = syncManager;
}

SyncThread.prototype = {
    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsIRunnable) ||
            iid.equals(Components.interfaces.nsISupports))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    run: function() {
        if (this.syncManager.systemSyncEnabled) {
            startSystemSync(this.syncManager);
        } else {
            FW_LOG("system sync disabled, call systemSyncReady method")
            systemSyncReady(this.syncManager);
        }

        //var mainThreadWorker = new MainThreadWorker();
        //this.syncManager.mainThread.dispatch(mainThreadWorker, this.syncManager.mainThread.DISPATCH_NORMAL);
    }
};

function MainThreadWorker(action, params) {
    this.action = action;
    this.params = params;
}

MainThreadWorker.prototype = {
    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsIRunnable) ||
            iid.equals(Components.interfaces.nsISupports))
            return this;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    run: function() {
        if (this.action) {
            FW_LOG("start main thread action: " + this.action);
            if (this.action == 'refreshCalendars') {
                refreshAllOXCalendars();
            } else if (this.action == 'startAddressbookSync') {
                var syncManager = this.params[0];
                syncManager.startAddressbookSync();
            }
        } else {
            FW_LOG("start main thread without action, just a callback");
        }
    }
};

function startMainThreadWorkerWithAction(mainThread, action, params) {
    var mainThreadWorker = new MainThreadWorker(action, params);
    mainThread.dispatch(mainThreadWorker, mainThread.DISPATCH_NORMAL);
}

function refreshAllOXCalendars() {
    // refresh all calendars
    var calendars = getCalendarManager().getCalendars({});

    for each (var calendar in calendars) {
        if (calendar.type == "oxcalendar") {
            calendar.refresh();
        }
    }
}

/*
 * Callback Methods
 */
function systemSyncReady(syncManager) {
    FW_LOG("systemSyncReady");

    updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.system-sync-finished'));

    FW_LOG("use synced folders for calendar synchronisation")
    var folderDict = getAllFolderDetails(syncManager.connectionDB);
    for (var folderId in folderDict) {
        syncManager.folders.push(folderId);
    }

    var defaultCalendarFolder = getOXCalendarPref(syncManager.connectionDB, "calendar_default_folder");
    if (defaultCalendarFolder) {
        syncManager.folders.push(defaultCalendarFolder);
    }

    FW_LOG("found " + syncManager.folders.length + " calendar folders for local storage");

    syncManager.startCalendarSync();
}

function calendarSyncReady(syncManager) {
    FW_LOG("calendarSyncReady: refresh all OX calendar");

    updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.calendar-sync-finished'));

    //FIXME disabled THREADING (see comment above)
//    if (isTB3) {
//        startMainThreadWorkerWithAction(syncManager.mainThread, 'refreshCalendars');
//        startMainThreadWorkerWithAction(syncManager.mainThread, 'startAddressbookSync', [ syncManager ]);
//    } else {
        refreshAllOXCalendars();
        syncManager.startAddressbookSync();
//    }
}

function addressbookSyncReady(syncManager) {
    FW_LOG("finished addressbook sync");
    updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.addressbook-sync-finished'));

    var statusManager = Components.classes['@fusionworkx.com/common/statusmanager;1'].getService().wrappedJSObject;
    statusManager.deleteStatus("addrSync");

    syncManager.addressbookSyncRunning = false;

    syncReady(syncManager);
}

function syncReady(syncManager) {
    syncManager.syncRunning = false;
    syncManager.notifyReadyCallbacks();

    // cleanup old session
    logoutOldSessions(syncManager.HOST, syncManager.connectionDB);

    updateStatusBarWithThread(getI18nString('oxcalendar-provider.statusbar.sync-finished'), "font-weight:bold;color:#006600");
}
