/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

var defaultItipTransport = Components.classes["@mozilla.org/calendar/itip-transport;1?type=email"].getService(Components.interfaces.calIItipTransport);

var calendarItipTransport = {

    mail: "",

    get defaultIdentity() {
        return this.mail;
    },

    get scheme() {
        return "mailto";
    },

    get senderAddress() {
        return this.mail;
    },

    set senderAddress(aValue) {
        this.mail = aValue;
    },

    get type() {
        return "email";
    },


    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.calIItipTransport)) {
            return this;
        }
    	throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    simpleSendResponse: function(item) {
        FW_LOG("simpleSendResponse not implemented");
    },

    sendItems: function(count, recipientArray, title, altrep, item) {
        FW_LOG("send items disabled, item for user: " + this.mail + " are send by the server");
    },

    checkForInvitations:function(searchStart) {
        FW_WARN("checkForInvitations currently not supported");
        return;
    }
};

function initOXCalendar(calendar) {
    FW_LOG("create new calOXCalendar instance");

    calendar.mapping = { };

    calendar.hasRemoteTrigger = true;

    calendar.enableRefresh = true;

    calendar.connectionDB = getOXCalendarDBConnection();

    calendar.observer_init();
}

function adoptCalendarItem(calendar, aItem, aListener) {
	FW_LOG("adoptCalendarItem");

	/*
	 * WORKAROUND:
	 * maybe this is a lightning problem. In some cases the aItem.calendar
	 * attribute is not set so we just set it with the given calendar parameter
	 */

	if (!aItem.calendar) {
		FW_WARN("new item doesn't have 'calendar' attribute. Use selected calendar");
		aItem.calendar = calendar;
	}

	readOnlyCheck(calendar, aListener);

	if (aListener != null) {
        var connection = calendar.connectionDB;
        var newestSession = getNewestSession(connection);
        calendar.remoteHandler.setSession(newestSession[0], newestSession[1]);
		calendar.remoteHandler.validateSession(calendar.HOST, calendar.USER, calendar.PASSWORD, validateSessionReady, [ 'add', aItem, aListener ], false);
	}
}

function genericCalendarAdoptItem(calendar, aItem, aListener) {
	FW_LOG("genericCalendarAdoptItem");

	var calendarId = calendar.mID;
    if (!calendarId) {
        calendarId = calendar.id;
    }

	var connection = calendar.connectionDB;
	var folder = getFolderFromCalendar(connection, calendar);

	try {
        //LOGitem(aItem);

        var data = item2object(aItem);

		if (aItem.id) {
            // if the item already has an it we try to use the uuid for open-xchange too
			FW_LOG("item has already an id (" + aItem.id + "). This is normally a invitation");
		} else {
            aItem.id = getUUID();
        }

        // send the uid to server to have a unique identifier for all actions
        data['uid'] = aItem.id;

        var isItipItem = aItem.getProperty('FX_ITIP_ITEM') == '1';
        if (isItipItem) {
            // don't send a notification if this item comes from an invitation
            data['notification'] = false;

            aItem.deleteProperty('FX_ITIP_ITEM');
        }

		addParticipants2Data(aItem, data, connection, 0);
		addConfirmation2Data(aItem, data, connection);

		var saveData = cloneObject(data);
		var remoteId = calendar.remoteHandler.insertAppointment(calendar.HOST, saveData, { });

		if (!(remoteId && remoteId > 0)) {
			FW_WARN("insert appointment doesn't return an object id from server");
			return;
		}
			
		data['object_id'] = remoteId;
			
		LOG("add new object to server with id: " + aItem.id);

		var localId = aItem.id;

		// add mapping for localId and remoteId
		insertCalendarIdMapping(connection, folder, localId, 0, remoteId, folder, 0);

		// insert item to internal storage and cache
		saveItem2LocalStoreAndCache(calendar.connectionDB, calendarId, folder, localId, remoteId, aItem, data);
        	
		aListener.onOperationComplete(calendar.superCalendar,
		Components.results.OK,
		Components.interfaces.calIOperationListener.ADD,
		aItem,
		aItem);
		calendar.observers.notify("onAddItem", [aItem]);
	} catch (e) {
		FW_ERROR("cannot add item: " + e);
		FW_LOG("item:");
		LOGitem(aItem);
	   	        
		throw e;
				
		// TODO: find out how to throw an error message to the UI
		/*
    		    aListener.onOperationComplete(this.superCalendar,
                                          	  e.result,
                                          	  Components.interfaces.calIOperationListener.ADD,
                                          	  null,
                                          	  e.message);
		 */
	}
}

function modifyCalendarItem(calendar, aNewItem, aOldItem, aListener) {
    FW_LOG("modifyCalendarItem");
	readOnlyCheck(calendar, aListener);
    
	if (aListener != null) {
        var connection = calendar.connectionDB;
        var newestSession = getNewestSession(connection);
        calendar.remoteHandler.setSession(newestSession[0], newestSession[1]);
		calendar.remoteHandler.validateSession(calendar.HOST, calendar.USER, calendar.PASSWORD, validateSessionReady, [ 'update', aNewItem, aOldItem, aListener ], false);
		//this.genericUpdateItem(aNewItem, aOldItem, aListener);
	}
}

function genericCalendarUpdateItem(calendar, aNewItem, aOldItem, aListener) {
   	var localId = aOldItem.id;
	var calendarId = calendar.mID;
    if (!calendarId) {
        calendarId = calendar.id;
    }

	var connection = calendar.connectionDB;
	var folder = getFolderFromCalendar(connection, calendar);

	LOG("send modified object to server with old id: " + localId + " or new id:" + aNewItem.id);
	var syncObject = getCalendarSyncObjectByLocalId(calendar.connectionDB, folder, localId);
	if (!syncObject) {
		FW_WARN("cannot load sync object by local id: " + localId);
      			
		aListener.onOperationComplete(calendar.superCalendar,
		Components.results.NS_OK,
		Components.interfaces.calIOperationListener.MODIFY,
		aNewItem.id,
		aNewItem);

        return;
	}

	var remoteId = syncObject['remote_id'];
    var parentRemoteId = remoteId;
	var folder = getFolderFromCalendar(connection, calendar);

	var new_data = item2object(aNewItem);

    var isException = false;

    var recurrenceDatePosition;

    if (aNewItem.recurrenceId) {
        FW_LOG("recurring exception detected, try to load parent object by id: " + aNewItem.recurrenceId);

        var recurrenceDatePosition = new Date(0);
        recurrenceDatePosition.setUTCFullYear(aNewItem.recurrenceId.year);
        recurrenceDatePosition.setUTCMonth(aNewItem.recurrenceId.month);
        recurrenceDatePosition.setUTCDate(aNewItem.recurrenceId.day);

        FW_LOG("use recurrence date position for recurrence update: " + recurrenceDatePosition);

        // for a recurring exception add the ox fields recurring_position
        // or recurring_date_position

        new_data['recurrence_date_position'] = recurrenceDatePosition.getTime();
        isException = true;
    }

    //FW_LOG("update appointment: " + new_data.toSource());
    //LOGitem(aNewItem);
       		
	try {
        // check if this modification is a recurring exception delete
        // this is only possible if the appointment is not an exception
        if (!isException && aOldItem.recurrenceInfo && aNewItem.recurrenceInfo) {
            // check if new item contains more exception after the delete
            if (aOldItem.recurrenceInfo.countRecurrenceItems() < aNewItem.recurrenceInfo.countRecurrenceItems()) {
                FW_LOG("recurrence item count was changed, check if it is a delete exception call");

                             
                //LOGitem(aNewItem);

                var loadDeleteExceptionParentData = loadAppointmentData(calendar.connectionDB, folder, localId);

                var deleteExceptionDateArray = { };

                var recurrenceItems = aNewItem.recurrenceInfo.getRecurrenceItems({});

                for each (var recurrenceItem in recurrenceItems) {
                    var recurrenceItemClone = recurrenceItem.clone();
                    var icalProperty = recurrenceItemClone.icalProperty;
                    if (icalProperty.propertyName != 'EXDATE') {
                        continue;
                    }

                    var deleteExceptionDate = icalProperty.valueAsDatetime;

                    var recurrenceDeleteDatePosition = new Date(0);
                    recurrenceDeleteDatePosition.setUTCFullYear(deleteExceptionDate.year);
                    recurrenceDeleteDatePosition.setUTCMonth(deleteExceptionDate.month);
                    recurrenceDeleteDatePosition.setUTCDate(deleteExceptionDate.day);

                    // do the delete exception stuff and finish this method

                    FW_LOG("found delete exception: " + recurrenceDeleteDatePosition);

                    var recurrenceDeleteDateAsLong = recurrenceDeleteDatePosition.getTime();
                    var deleteAppointmentData = { 'id': remoteId, 'recurrence_date_position': recurrenceDeleteDateAsLong };

                    calendar.remoteHandler.deleteAppointment(calendar.HOST, deleteAppointmentData, { });

                    deleteExceptionDateArray.push(recurrenceDeleteDateAsLong);
                }

                if (deleteExceptionDateArray.length > 0) {
                    // save deleted exceptions to locale storage

                    loadDeleteExceptionParentData['delete_exceptions'] = deleteExceptionDateArray;
                    saveItem2LocalStoreAndCache(calendar.connectionDB, calendarId, folder, localId, remoteId, aNewItem, loadDeleteExceptionParentData);                
                } else {
                    FW_LOG("no deleted exceptions found");
                }

                aListener.onOperationComplete(calendar.superCalendar,
                Components.results.NS_OK,
                Components.interfaces.calIOperationListener.MODIFY,
                aNewItem.id,
                aNewItem);

                calendar.observers.notify("onModifyItem", [aNewItem, aOldItem]);

                // UGLY WORKAROUND: for some reasons I have to refresh the calendar because the complete
                // recurrence appointment is deleted from the UI after deleting a single recurrence.
                calendar.refresh();

                return;
            }
        }

        addParticipants2Data(aNewItem, new_data, calendar.connectionDB, 0, calendar.ORGANIZER);
       		       		
        var old_data = item2object(aOldItem);
        addParticipants2Data(aOldItem, old_data, calendar.connectionDB, 0, calendar.ORGANIZER);
           	
        var isEquals = isAppointmentEqual(aNewItem, aOldItem);
           	
        if (!isEquals) {
            addConfirmation2Data(aNewItem, new_data, calendar.connectionDB);
					
            new_data['id'] = remoteId;
            var saveData = cloneObject(new_data);
					
            var newObjectId = calendar.remoteHandler.updateAppointment(calendar.HOST, saveData, { });
            if (!(newObjectId && newObjectId > 0)) {
                FW_WARN("update appointment doesn't return an object id from server");
                return;
            }
        		
            remoteId = newObjectId;
				
            delete new_data['id'];

            if (isException) {
                new_data['object_id'] = remoteId;

                // update parent object
                var loadParentData = loadAppointmentData(calendar.connectionDB, folder, localId);
                var recurrenceExceptions = loadParentData['change_exceptions'];
                if (!recurrenceExceptions) {
                    recurrenceExceptions = [ ];
                }

                recurrenceExceptions.push(recurrenceDatePosition);

                loadParentData['change_exceptions'] = recurrenceExceptions;

                saveItem2LocalStoreAndCache(calendar.connectionDB, calendarId, folder, localId, remoteId, aOldItem, loadParentData);
            } else {
                new_data['object_id'] = parentRemoteId;
            }

            // update item in internal storage and cache
            saveItem2LocalStoreAndCache(calendar.connectionDB, calendarId, folder, localId, remoteId, aNewItem, new_data);

            FW_LOG("object with id " + aNewItem.id + " modified and mapping for remote id " + remoteId + " exists");
        
            aListener.onOperationComplete(calendar.superCalendar,
            Components.results.NS_OK,
            Components.interfaces.calIOperationListener.MODIFY,
            aNewItem.id,
            aNewItem);
        }
    } catch (e) {
        FW_ERROR("cannot modify item: " + e);

        FW_LOG("old item:");
        LOGitem(aOldItem);
        FW_LOG("new item:");
        LOGitem(aNewItem);
	   	        
        throw e;
    }
        
    calendar.observers.notify("onModifyItem", [aNewItem, aOldItem]);
}

function saveItem2LocalStoreAndCache(connection, calendarId, folderId, localId, remoteId, item, appointmentObj) {
    var syncObject = getCalendarSyncObjectByRemoteId(connection, calendarId, remoteId);

    var startAndEnd = parseStartAndEndFromAppointment(appointmentObj);
    var start = startAndEnd[0];
    var end = startAndEnd[1];

    var appointmentObjAsString = FXJSON.toString(appointmentObj);

    saveAppointmentData(connection, folderId, localId, start, end, appointmentObjAsString);

    var iCacheManager = Components.classes['@fusionworkx.com/common/cachemanager;1'].getService().wrappedJSObject;
    iCacheManager.setItem2Cache(localId, item);
}

function deleteCalendarItem(calendar, aItem, aListener) {
    FW_LOG("deleteCalendarItem");

    readOnlyCheck(calendar, aListener);

    if (aListener != null) {
        var connection = calendar.connectionDB;
        var newestSession = getNewestSession(connection);
        calendar.remoteHandler.setSession(newestSession[0], newestSession[1]);
        calendar.remoteHandler.validateSession(calendar.HOST, calendar.USER, calendar.PASSWORD, validateSessionReady, [ 'delete', aItem, aListener ], false);
        //this.genericDeleteItem(aItem, aListener);
    }
}

function genericCalendarDeleteItem(calendar, aItem, aListener) {
    var localId = aItem.id;
    var connection = calendar.connectionDB;
    var folder = getFolderFromCalendar(connection, calendar);
    var calendarId = calendar.mID;
    if (!calendarId) {
        calendarId = calendar.id;
    }

    var syncObject = getCalendarSyncObjectByLocalId(connection, folder, localId);
    if (!syncObject) {
        FW_WARN("cannot load sync object by local id: " + localId);

        aListener.onOperationComplete(calendar.superCalendar,
        Components.results.NS_OK,
        Components.interfaces.calIOperationListener.DELETE,
        aItem.id,
        aItem);
    }

    var remoteId = syncObject['remote_id'];
    var appointmentData = { 'id': remoteId };

    calendar.remoteHandler.deleteAppointment(calendar.HOST, appointmentData, { });

    // delete item from internal storage and from cache
    deleteAppointmentData(connection, folder, localId);

    var iCacheManager = Components.classes['@fusionworkx.com/common/cachemanager;1'].getService().wrappedJSObject;
    iCacheManager.deleteItemFromCache(localId);
    // done

    // delete mapping for localId
    deleteCalendarIdMapping(connection, folder, [ localId ]);

    aListener.onOperationComplete(calendar.superCalendar,
    Components.results.NS_OK,
    Components.interfaces.calIOperationListener.DELETE,
    aItem.id,
    aItem);
    calendar.observers.notify("onDeleteItem", [aItem]);
}

function getCalendarItem(calendar, aId, aListener) {
    FW_LOG("getCalendarItem: aId=" + aId);

    var connection = calendar.connectionDB;
    var calendarId = calendar.mID;
    if (!calendarId) {
        calendarId = calendar.id;
    }

    var folder = getFolderFromCalendar(connection, calendar);

    if (aListener != null) {
        FW_LOG("get item with id: " + aId);
			
        var syncObject = getCalendarSyncObjectByLocalId(connection, folder, aId);
        if (!syncObject) {
            FW_LOG("no id mapping found for id: " + aId);

            aListener.onGetResult(calendar.superCalendar, Components.results.NS_OK, Components.interfaces.calIEvent, null, 0, null);
			
            aListener.onOperationComplete(calendar.superCalendar,
            Components.results.NS_OK,
            Components.interfaces.calIOperationListener.GET,
            null,
            null);
            return;
        }

        var participantDetails = getAllParticipantDetails(connection);
        var appointment = loadAppointmentData(connection, folder, aId);
        if (appointment) {
            FW_LOG("getItem: requested appointment found in internal storage found");

            var item = createEvent();
            item.calendar = calendar;
            item = object2ItemHelper(calendar, item, appointment, participantDetails);
            item.id = aId;
			
            aListener.onGetResult(calendar.superCalendar, Components.results.NS_OK, Components.interfaces.calIEvent, null, 1, [ item ]);
			
            aListener.onOperationComplete(calendar.superCalendar,
            Components.results.NS_OK,
            Components.interfaces.calIOperationListener.GET,
            aId,
            null);
        } else {
            FW_LOG("getItem: requested item not found");

            aListener.onGetResult(calendar.superCalendar, Components.results.NS_OK, Components.interfaces.calIEvent, null, 0, null);
			
            aListener.onOperationComplete(calendar.superCalendar,
            Components.results.NS_OK,
            Components.interfaces.calIOperationListener.GET,
            null,
            null);
        }
    }
}

function getCalendarItems(calendar, aItemFilter, aCount, aRangeStart, aRangeEnd, aListener) {
    // ENABLE THIS FOR DEBUGGING TO PREVENT MUCH CALENDAR LOGGING INFOS
    /*
    aListener.onOperationComplete(calendar.superCalendar,
    Components.results.NS_OK,
    Components.interfaces.calIOperationListener.GET,
    null,
    null);

    return;
     */
                                    
    FW_LOG("getItems call: aCount: " + aCount + " start: " + aRangeStart + " end: " + aRangeEnd + " item filter: " + aItemFilter);

    if (!aListener) {
        return;
    }
        
    var wantEvents = ((aItemFilter &
        Components.interfaces.calICalendar.ITEM_FILTER_TYPE_EVENT) != 0);

    if (!wantEvents) {
        FW_LOG("something other than event items requested, ignoring request");
	
        aListener.onOperationComplete(calendar.superCalendar,
        Components.results.NS_OK,
        Components.interfaces.calIOperationListener.GET,
        null,
        null);

        return;
    }

    try {
        var connection = calendar.connectionDB;
        var folder = getFolderFromCalendar(connection, calendar);

        var participantDetails = getAllParticipantDetails(connection);

        var rangeStartAsLong;
        var rangeEndAsLong;

        if (aRangeStart) {
            rangeStartAsLong = dateTime2Date(aRangeStart).getTime() - (12*60*60*1000);
        } else {
            // if end is null use end date - 1 years
            rangeStartAsLong = new Date().getTime() - (365*24*60*60*1000);;
        }

        if (aRangeEnd) {
            rangeEndAsLong = dateTime2Date(aRangeEnd).getTime() + (12*60*60*1000);
        } else {
            // if end is null use end date + 1 years
            rangeEndAsLong = new Date().getTime() + (365*24*60*60*1000);
        }

        var appointments = listAppointmentsInRange(connection, folder, rangeStartAsLong, rangeEndAsLong);

        var iCacheManager = Components.classes['@fusionworkx.com/common/cachemanager;1'].getService().wrappedJSObject;

        convertOXCalendar(calendar, iCacheManager, appointments, participantDetails, aListener, aCount, aRangeStart, aRangeEnd, aItemFilter);
    } catch (e) {
        FW_ERROR("cannot read appointments from local storage: " + e);

        aListener.onOperationComplete(calendar.superCalendar,
        e.result,
        Components.interfaces.calIOperationListener.GET,
        null,
        e.message);
    }
}

function calendarRefresh(calendar) {
    calendar.ORGANIZER = calendar.getProperty("organizerId");

    if (!calendar.connectionDB) {
        calendar.connectionDB = getOXCalendarDBConnection();
    }

    if (!calendar.HOST) {
        calendar.HOST = getOXCalendarPref(calendar.connectionDB, "host");
    }

    var username = getOXCalendarPref(calendar.connectionDB, "username");
    var password = getOXCalendarPref(calendar.connectionDB, "password");

    calendar.USER = username;
    calendar.PASSWORD = password;

    var syncManager = {
        'connectionDB': calendar.connectionDB,
        'HOST': calendar.HOST
    };

    var folderId = getFolderFromCalendar(calendar.connectionDB, calendar);

    if (!calendar.remoteHandler) {
        FW_LOG("calendar remote handler not set, set default remote handler for folder: " + folderId);
        var serverType = getOXCalendarPref(calendar.connectionDB, "server-type");

        calendar.remoteHandler = new calRemoteHandler();

        calendar.ox6Handler = new calOX6RemoteHandler(syncManager);

        calendar.remoteHandler.registerHandler("OX6", calendar.ox6Handler);

        FW_LOG("set remote handler for server type: " + serverType);
        calendar.remoteHandler.setHandler(serverType);

        FW_LOG("add calendar to remote handler");
        calendar.remoteHandler.setCalendar(calendar);
    }

    FW_LOG("set folder property for remote handler: " + folderId);
    calendar.ox6Handler.calendarFolder = folderId;
}

function convertOXCalendar(calendar, iCacheManager, appointment_array, participantDetails, aListener, aCount, aRangeStart, aRangeEnd, aItemFilter) {
    FW_LOG("convert appointment_array to calendar item, found " + appointment_array.length + " objects");
    var wantEvents = ((aItemFilter &
        Components.interfaces.calICalendar.ITEM_FILTER_TYPE_EVENT) != 0);

    if (wantEvents) {
        var cacheCounter = 0;
        
        for (var a = 0; a < appointment_array.length; a++) {
            var event = appointment_array[a];

            var objectId = event['object_id'];
            var localId = event['local_id'];

            // OX Offset conversion
            FW_LOG("start object to item conversion for object id: " + objectId);
            	
            var item = createEvent();
            item.calendar = calendar;

            var cachedItem = iCacheManager.getItemFromCache(localId);
            if (cachedItem) {
                cacheCounter++;
                item = cachedItem;
            } else {
                try {
                    //FW_LOG("parse item: " + event['title']);
                    item = object2ItemHelper(calendar, item, event, participantDetails);
                    iCacheManager.setItem2Cache(localId, item);
                } catch (e) {
                    FW_ERROR("cannot convert object 2 item: " + e);

                    FW_LOG("event: " + event.toSource());
                    continue;
                }
				
                item.id = localId;
            }
    			
            if (item.recurrenceInfo) {
                FW_LOG("item " + item.title + " converted, try to expand recurring items");
                var expandedItems = expandItems(item, aItemFilter, aRangeStart, aRangeEnd);
                FW_LOG("recurrence item expanded");;
                for (var b = 0; b < expandedItems.length; b++) {
                    var expItem = expandedItems[b];

                    //var isException = isItemAnException(expItem, changeExceptions, deleteExceptions, event.title);
                    //if (!isException) {
                    sendListenerEvent(this, expItem, aRangeStart, aRangeEnd, objectId, aListener);
                    //} else {
                    //sendListenerEvent(null, aRangeStart, aRangeEnd, null, aListener);
                    //}
                }
					
            } else {
                FW_LOG("send event listener");
                sendListenerEvent(this, item, aRangeStart, aRangeEnd, objectId, aListener);
            }
        }
            
        FW_LOG(appointment_array.length + " items found, read " + cacheCounter + " items from internal cache");
    }

    aListener.onOperationComplete(calendar.superCalendar,
    Components.results.NS_OK,
    Components.interfaces.calIOperationListener.GET,
    null,
    null);
}
    
function sendListenerEvent (calendar, item, rangeStart, rangeEnd, objectId, aListener) {
    // method from calUtils
    if (!checkIfInRange(item, rangeStart, rangeEnd)) {
        FW_LOG("item: " + objectId + " not in range");
        //FW_LOG("item range: item.startDate=" + item.startDate + ", item.endDate=" + item.endDate + ", rangeStart=" + rangeStart + ", rangeEnd=" + rangeEnd);
        return;
    }

    item.makeImmutable();
    aListener.onGetResult(calendar.superCalendar, Components.results.NS_OK, Components.interfaces.calIEvent, null, 1, [item]);
}

function object2ItemHelper(calendar, item, appointment, participantDetails) {
    var connection = calendar.connectionDB;

    var users = appointment['users'];
    if (users && users.length > 0) {
        addConfirmation2Item(item, users, connection);
    } else {
        item.status = "CONFIRMED";
    }

    var participants = appointment['participants'];
    var attendee_array = [ ];

    if (participants && participants.length > 0) {
        item.removeAllAttendees();

        attendee_array = participants2Attendees(participants, participantDetails, appointment.created_by, calendar.ORGANIZER)

        // check if an organizer exists and add this organizer as item organizer
        for (var b = 0; b < attendee_array.length; b++) {
            var attendee = attendee_array[b];
            if (attendee.isOrganizer) {
                item.organizer = attendee;
            }
        }

        if (!item.organizer) {
            FW_LOG("no organizer found, add calendar organizer: " + calendar.ORGANIZER + " as organizer");

            var organizerAttendee = createAttendee();
            organizerAttendee.isOrganizer = true;
            organizerAttendee.id = calendar.ORGANIZER;
            organizerAttendee.role = "REQ-PARTICIPANT";
            organizerAttendee.userType = "INDIVIDUAL";
            organizerAttendee.makeImmutable();

            item.organizer = organizerAttendee;
        }
    }

    FW_LOG("item attendee converter finished");

    item = object2item(item, appointment, attendee_array);

    return item;
}
    
function readOnlyCheck(calendar, aListener) {
    if (calendar.readOnly) {
        calendar.notifyOperationComplete(aListener,
        Components.interfaces.calIErrors.CAL_IS_READONLY,
        Components.interfaces.calIOperationListener.MODIFY,
        null,
        "Calendar is readonly");
    }
}

function expandItems(aItem, aItemFilter, aRangeStart, aRangeEnd) {
    var expandedItems;

    try {
        if (aItemFilter & Components.interfaces.calICalendar.ITEM_FILTER_CLASS_OCCURRENCES) {
            expandedItems = aItem.getOccurrencesBetween(aRangeStart,
            aRangeEnd, { });
            FW_LOG("Expanded item " + aItem.title + " to " +
                expandedItems.length + " items");
        }
    } catch (e) {
        FW_WARN("cannot expand recurrence item: " + e);
        LOGitem(aItem);
    }
    return expandedItems || [aItem];
}

function isItemAnException(expItem, changeExceptions, deleteExceptions, title) {
    //removeOccurrenceAt (in calIDateTime aRecurrenceId)
    var recurrenceId = expItem.recurrenceId;
    var recurrenceIdAsDate = dateTime2Date(recurrenceId);

    var recurrenceIdAsLong = Date.UTC(recurrenceIdAsDate.getFullYear(), recurrenceIdAsDate.getMonth(), recurrenceIdAsDate.getDate(), 0, 0, 0);

    if (changeExceptions) {
        //FW_LOG("changeException.length: " + changeExceptions.length);
        for (var a = 0; a < changeExceptions.length; a++) {
            FW_LOG("changeException: " + changeExceptions[a] + " recurrenceIdAsLong: " + recurrenceIdAsLong);
            if (changeExceptions[a] == recurrenceIdAsLong) {
                return true;
            }
        }
    }

    if (deleteExceptions) {
        //FW_LOG("deleteException.length: " + deleteExceptions.length);
        for (var a = 0; a < deleteExceptions.length; a++) {
            FW_LOG("deleteException: " + deleteExceptions[a] + " recurrenceIdAsLong: " + recurrenceIdAsLong);
            if (deleteExceptions[a] == recurrenceIdAsLong) {
                return true;
            }
        }
    }
	
    return false;
}

// CALLBACK METHODS
function validateSessionReady(calendar, success, params) {
    FW_LOG("validate session ready event detected for calendar " + calendar.name + " from host " + calendar.HOST);
    var action = params[0];
    
    FW_LOG("call action method on calendar: " + action);

    if (action == 'add') {
        genericCalendarAdoptItem(calendar, params[1], params[2]);
    } else if (action == 'update') {
        genericCalendarUpdateItem(calendar, params[1], params[2], params[3]);
    } else if (action == 'delete') {
        genericCalendarDeleteItem(calendar, params[1], params[2]);
    }
}

/*
 * commonCalendarObserver
 * detected events send from the GUI to the calendar
 */

function commonCalendarObserver(topic, calendar)
{
    this.topic = topic;
    this.calendar = calendar;
    this.register();
}

commonCalendarObserver.prototype = {
    observe: function(subject, topic, data) {
        FW_LOG("got notify event from GUI: topic=" + topic + ", data=" + data);
        FW_LOG("refresh event detected for calendar " + this.calendar.name);
        this.calendar.refresh();
    },
  	
    register: function() {
        var observerService = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
        observerService.addObserver(this, this.topic, false);
    },
  	
    unregister: function() {
        var observerService = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
        observerService.removeObserver(this, this.topic);
    }
}

// TODO: anderer Methoden name!?
function isAppointmentEqual(data1, data2) {
    // TODO: add real equals check
    if (true) {
        return false;
    }

    var fields = [ 'title', 'location', 'description', 'start_date', 'end_date'];
	
    if (data1['participants']) {
        return false;
    }
	
    if (data1.length != data2.length) {
        return false;
    }

    for (var a = 0; a < fields.length; a++) {
        if (data1[fields[a]] && data2[fields[a]]) {
            if (data1[fields[a]] != data2[fields[a]]) {
                return false;
            }
        } else {
            return false;
        }
    }
	
    return true;
}

function endsWith(search, pattern) {
    return (search.match(pattern+"$")==pattern);
}