/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

var FUSIONSYNC_ID = "fusionsyncId";

var contactFieldMapping = {
	'display_name' : 'DisplayName',
	'first_name' : 'FirstName',
	'last_name' : 'LastName',
	'email1' : 'PrimaryEmail',
	'email2' : 'SecondEmail',
	'nickname' : 'NickName',
	'telephone_business1' : 'WorkPhone',
	'telephone_home1' : 'HomePhone',
	'fax_business' : 'FaxNumber',
	'telephone_pager' : 'PagerNumber',
	'cellular_telephone1' : 'CellularNumber',
	'street_home' : 'HomeAddress',
	'city_home' : 'HomeCity',
	'state_home' : 'HomeState',
	'postal_code_home' : 'HomeZipCode',
	'country_home' : 'HomeCountry',
	'street_business' : 'WorkAddress',
	'city_business' : 'WorkCity',
	'state_business' : 'WorkState',
	'postal_code_business' : 'WorkZipCode',
	'country_business' : 'WorkCountry',
	'position' : 'JobTitle',
	'department' : 'Department',
	'company' : 'Company',
	'note' : 'Notes',
	'url' : 'WebPage1',
	'birth_year' : 'BirthYear',
	'birth_month' : 'BirthMonth',
	'birth_day' : 'BirthDay',
	'anniversary_year' : 'AnniversaryYear',
	'anniversary_month' : 'AnniversaryMonth',
	'anniversary_day' : 'AnniversaryDay'
};

function getAddressbookURI(addressbook) {

	return addressbook.URI;

}

function getAddressbookDirectory(uri) {
	FW_LOG("getAddressbookDirectory: uri=" + uri);

	var addressbookDirectory;

	var abManager = Components.classes["@mozilla.org/abmanager;1"]
			.getService(Components.interfaces.nsIAbManager);
	FW_LOG("found addressbook manager, uri: " + uri);
	addressbookDirectory = abManager.getDirectory(uri);
	FW_LOG("got addressbook directory by uri: " + addressbookDirectory.URI);

	return addressbookDirectory;
}

function getUUID() {
	var uuidGen = Components.classes["@mozilla.org/uuid-generator;1"]
			.getService(Components.interfaces.nsIUUIDGenerator);
	return uuidGen.generateUUID().toString().replace(/[{}]/g, '');
}

function oxContact2Card(oxContact, addressbookCard) {
	// prepare special fields

	if (oxContact['birthday']) {
		var birthdayDate = new Date(oxContact['birthday']);
		FW_LOG("birthdayDate: " + birthdayDate);
		oxContact['birth_year'] = birthdayDate.getUTCFullYear();
		oxContact['birth_month'] = birthdayDate.getUTCMonth() + 1;
		oxContact['birth_day'] = birthdayDate.getUTCDate();
	}

	if (oxContact['anniversary']) {
		var anniversaryDate = new Date(oxContact['anniversary']);
		FW_LOG("anniversaryDate: " + anniversaryDate);
		oxContact['anniversary_year'] = anniversaryDate.getUTCFullYear();
		oxContact['anniversary_month'] = anniversaryDate.getUTCMonth() + 1;
		oxContact['anniversary_day'] = anniversaryDate.getUTCDate();
	}

	for (oxField in contactFieldMapping) {
		var cardField = contactFieldMapping[oxField];
		// FW_LOG("cardField: " + cardField + " value: " + oxContact[oxField]);

		if (oxContact[oxField] && oxContact[oxField] != null) {
			var value = oxContact[oxField];

			addressbookCard.setProperty(cardField, value);

		} else {

			addressbookCard.setProperty(cardField, "");

		}
	}

	return addressbookCard;
}

function card2OXContact(addressbookCard) {
	var oxContact = {};

	for ( var oxField in contactFieldMapping) {
		var cardField = contactFieldMapping[oxField];
		var value;

		value = addressbookCard.getProperty(cardField, "");

		if (value) {
			oxContact[oxField] = value;
		} else {
			oxContact[oxField] = "";
		}
	}

	if (oxContact['birth_year'] && oxContact['birth_month']
			&& oxContact['birth_day']) {
		var birthdayDate = new Date(0);
		birthdayDate.setUTCFullYear(oxContact['birth_year']);
		birthdayDate.setUTCMonth(oxContact['birth_month'] - 1);
		birthdayDate.setUTCDate(oxContact['birth_day']);

		oxContact['birthday'] = birthdayDate.getTime();
	}

	if (oxContact['anniversary_year'] && oxContact['anniversary_month']
			&& oxContact['anniversary_day']) {
		var anniversaryDate = new Date(0);
		anniversaryDate.setUTCFullYear(oxContact['anniversary_year']);
		anniversaryDate.setUTCMonth(oxContact['anniversary_month'] - 1);
		anniversaryDate.setUTCDate(oxContact['anniversary_day']);

		oxContact['anniversary'] = birthdayDate.getTime();
	}

	return oxContact;
}

/*
 * Handy method to read displayName for TB2 and TB3
 */
function getCardDisplayName(card) {

	return card.getProperty("DisplayName", "");

}

/*
 * Handy method to read lastModified for TB2 and TB3
 */
function getCardLastModified(card) {

	return card.getProperty('LastModifiedDate', 0);

}