/* ***** BEGIN LICENSE BLOCK *****
 *
 *    Open-Xchange Calendar Provider
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *    Any modifications to this package must retain all copyright notices 
 *    of the original copyright holder(s) for the original code used.
 *    
 *    After any such modifications, the original code will still remain
 *    copyrighted by the copyright holder(s) or original author(s).
 *
 *     Copyright (C) 2008 - 2009 Sebastian Kauss | fusionworkx
 *     mail:	                 sebastian.kauss@fusionworkx.com
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the Free
 *     Software Foundation; either version 2 of the License, or (at your option)
 *     any later version.
 * 
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 * 
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

/*
 This file exists two times. The original files is in the /js/ directory and the copy is located
 in the directory /chrome/content/
 */

var offsetConverter = {
	'M' : 1,
	'H' : 60,
	'D' : 1440,
	'W' : 10080
};

var confirmationMapping = {
	'CONFIRMED' : 1,
	'CANCELLED' : 2,
	'TENTATIVE' : 3
};
var reverseConfirmationMapping = {
	0 : 'NEEDS-ACTION',
	1 : 'CONFIRMED',
	2 : 'CANCELLED',
	3 : 'TENTATIVE'
};

// OBSERVER HELPER
function notifyObservers(topic, data) {
	var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	if (observerService) {
		// observerService.notifyObservers(null, topic, data);
	} else {
		FW_WARN("cannot notify observer with topic " + topic);
	}
}

function getServerType(host) {
	FW_LOG("check OX version for host: " + host);
	var loginUrl = host + "/ajax/login?versionCheck=true";
	var request = getRequest();
	request.open("GET", loginUrl, false);
	request.overrideMimeType('text/plain');
	request.send(null);

	if (request.status == 200) {
		return "OX6";
	}

	var url = host + "/servlet/webdav.version";

	request = getRequest();
	request.open("GET", url, false);
	request.overrideMimeType('text/plain');
	request.send(null);

	if (request.status == 200) {
		FW_LOG("servlet url: /servlet/webdav.version exists");

		var text = request.responseText;
		FW_LOG("OX5 response text: " + text);

		if (text.match(/WebDAV:\s\d+/)) {
			// ox5 or ox6 version detected
			var regexPattern = /WebDAV:\s(\d+)/;
			regexPattern.exec(text);
			var majorVersion = String(RegExp.$1).substring(0, 1);

			FW_LOG("commercial ox5 or ox6 server found, check major version: " + majorVersion);

			if (majorVersion == "6") {
				return "OX6";
			} else if (majorVersion == "5") {
				return "OX5";
			}
		}

		FW_LOG("no commercial ox5 or ox6 server found, try to check ox5 community version");

		if (text.match(/WebDAV:\s\d.\d.\d/)) {
			// ox5 community version detected
			FW_LOG("OX5 community version detected");
			return "OX5"
		}

		return "UNKNOWN";
	} else {
		FW_ERROR("Unknown Server Type found for URL: " + url);
		return "UNKNOWN";
	}
}

function getRequest() {
	var request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIJSXMLHttpRequest);

	return request;
}

function item2object(item) {
	var title = item.title;

	FW_LOG("item2object: " + title);
	var startDate = dateTime2Date(item.startDate);
	var endDate = dateTime2Date(item.endDate);

	FW_LOG("item2object startDate: " + startDate);

	var data = {};
	data['title'] = title;

	var fullTime = item.startDate.isDate;
	if (fullTime) {
		data['full_time'] = fullTime;
	}

	data['start_date'] = date2utc_long(startDate);
	data['end_date'] = date2utc_long(endDate);

	var location = item.getProperty("LOCATION")
	if (location) {
		data['location'] = location;
	} else {
		data['location'] = "";
	}

	var description = item.getProperty("DESCRIPTION")
	if (description) {
		data['note'] = description;
	} else {
		data['note'] = "";
	}

	var transparency = item.getProperty("TRANSP");
	if (transparency) {
		FW_LOG("transparency: " + transparency);
		if (transparency == 'OPAQUE') {
			data['shown_as'] = 1;
		} else if (transparency == 'TRANSPARENT') {
			data['shown_as'] = 4;
		} else {
			data['shown_as'] = 1;
		}
	} else {
		data['shown_as'] = 1;
	}

	if (item.privacy) {
		FW_LOG("privacy: " + item.privacy);
		if (item.privacy == "CONFIDENTIAL" || item.privacy == "PRIVATE") {
			data['private_flag'] = true;
		} else {
			data['private_flag'] = false;
		}
	} else {
		data['private_flag'] = false;
	}

	var categories = item.getCategories({});
	if (categories) {
		data['categories'] = categories.join();
	} else {
		data['categories'] = "";
	}

	FW_LOG("item2object recurrence");
	if (item.recurrenceInfo) {
		FW_LOG("recurring appointment detected");

		var recurrenceDateArray = []
		var rrule = "";

		// var exids = item.recurrenceInfo.getExceptionIds({});
		// for each (var exc in exids) {
		// WARN(exc);
		// }

		var recurrenceItems = item.recurrenceInfo.getRecurrenceItems({});

		var counter = 0;
		for each ( var recurrenceItem in recurrenceItems) {
			var icalProperty = recurrenceItem.icalProperty;
			// parse EXTDATE
			if (calInstanceOf(recurrenceItem, Components.interfaces.calIRecurrenceDate)) {
				icalProperty.valueAsDatetime = recurrenceItem.date.getInTimezone(UTC());
			}

			var icalstring = icalProperty.icalString;

			if (icalstring.length >= 5 && icalstring.substring(0, 5) == 'RRULE') {
				addRecurrenceAttributes2Object(recurrenceItem, data);
			}
			counter++;
		}
	}

	var alarmOffset;

	var alarms = item.getAlarms({});
	if (alarms.length == 0) {
		FW_LOG("no alarms found in item");
		data['alarm'] = -1;
	} else {
		var alarm = alarms[0];
		alarmOffset = alarm.offset;

		FW_LOG("found alarm item: related=" + alarm.related + ", minutes=" + alarmOffset.minutes);

		if (alarm.related == Components.interfaces.calIItemBase.ALARM_RELATED_END) {
			FW_WARN("alarm for end date is not supported");
		} else {
			if (alarmOffset.minutes) {
				data['alarm'] = alarmOffset.minutes;
			} else {
				data['alarm'] = -1;
			}
		}
	}

	var fxDismissAlarm = item.getProperty("FX-DISMISS-ALARM");
	if (fxDismissAlarm && fxDismissAlarm == "TRUE") {
		data['alarm'] = -1;
		item.deleteProperty("FX-DISMISS-ALARM");
	}

	return data;
}

// CONFIRMATION HELPER METHODS
function addConfirmation2Data(item, data, connection) {
	var identifier = getOXCalendarPref(connection, "identifier");
	if (!identifier) {
		FW_WARN("no user identifier found for logged in user");
		return;
	}

	// check if identifier is organizer
	// (this is a workaround for OX5, OX6 sets the organizer status always to
	// confirmed)
	var isIdentifierOrganizer = false;

	var participant;

	if (item.organizer) {
		participant = attendee2participant(item.organizer, connection, 0);
		if (participant.id == identifier) {
			isIdentifierOrganizer = true;
		}
	}

	var confirmation = 0;

	if (isIdentifierOrganizer) {
		confirmation = 1;
	} else {
		// NEEDS-ACTION == NONE
		FW_LOG("addConfirmation2Data: read item status=" + item.status);
		if (confirmationMapping[item.status]) {
			confirmation = confirmationMapping[item.status];
		}
	}

	// iterate over all participants and set confirmation
	// for logged in user
	if (data.participants) {
		for ( var a = 0; a < data.participants.length; a++) {
			participant = data.participants[a];

			if (participant.id == identifier) {
				participant.confirmation = confirmation;
			}
		}
	}
}

function addConfirmation2Item(item, users, connection) {
	FW_LOG("addConfirmation2Item: user=" + users.toSource());

	var identifier = getOXCalendarPref(connection, "identifier");
	if (!identifier) {
		FW_WARN("no user identifier found for logged in user");
		return;
	}

	var confirmation = "NEEDS-ACTION";
	for ( var a = 0; a < users.length; a++) {
		var user = users[a];
		if (user.id == identifier) {
			confirmation = user.confirmation;
			if (!confirmation) {
				continue;
			}

			if (reverseConfirmationMapping[confirmation]) {
				confirmation = reverseConfirmationMapping[confirmation];
			}

			item.status = confirmation;

			FW_LOG("addConfirmation2Item: set item status=" + item.status);
			return;
		}
	}

	FW_LOG("user is not participant, don't set item status");
}

function addParticipants2Data(item, data, connection, calendarId, organizerId) {
	LOG("addParticipants2Data");
	var attendees = item.getAttendees({});

	// add organizer to normal participant list
	if (item.organizer) {
		FW_LOG("append organizer '" + item.organizer.id + "' to attendee list");
		attendees.push(item.organizer);
	} else {
		FW_LOG("addParticipants2Data: no item organizer found"); // , add
		// default
		// organizer:
		// " +
		// organizerId);
	}

	var participants = attendees2Participants(attendees, connection, calendarId);
	if (participants) {
		data['participants'] = participants;
	}
}

function attendees2Participants(attendees, connection, calendarId) {
	var participants = [];

	for each ( var attendee in attendees) {
		var participant = attendee2participant(attendee, connection, calendarId);
		if (participant) {
			FW_LOG("resolved attendee to participant: " + participant.toSource());
			participants[participants.length] = participant;
		} else {
			FW_WARN("cannot resolve attendee '" + attendee.id + "' to participant");
		}
	}

	if (participants.length > 0) {
		return participants;
	}

	return undefined;
}

function attendee2participant(attendee, connection, calendarId) {
	FW_LOG("attendee2participant=" + attendee + " role=" + attendee.role);

	var mail;
	if (attendee.id.indexOf("mailto:") == 0) {
		mail = attendee.id.substring(7, attendee.id.length);
	} else {
		FW_LOG("attendee doesn't starts with 'MAILTO': " + attendee.id);
		mail = attendee.id;
	}

	mail = mail.toLowerCase();

	if (!attendee.role) {
		FW_WARN("attendee " + attendee + " has no role, use default participant role");
		attendee.role = "REQ-PARTICIPANT";
	}

	if (attendee.role == "REQ-PARTICIPANT" || attendee.role == "CHAIR") {
		var mailprefix = mail;
		if (mailprefix.indexOf("@") != -1) {
			mailprefix = mailprefix.substring(0, mailprefix.indexOf("@"));
		}

		var groupId = getParticipantIdByAttribute(connection, calendarId, "mail", mailprefix, 2);
		if (groupId) {
			return {
				'id' : groupId,
				'type' : 2
			};
		}

		var userId = getParticipantIdByAttribute(connection, calendarId, "mail", mail, 1);
		if (userId) {
			return {
				'id' : userId,
				'type' : 1
			};
		}

		FW_LOG("participant still not resolved, try to resolve as resource by display name " + mail);
		var resourceId = getParticipantIdByAttribute(connection, calendarId, "display_name", mailprefix, 3);
		if (resourceId) {
			return {
				'id' : resourceId,
				'type' : 3
			};
		}

		resourceId = getParticipantIdByAttribute(connection, calendarId, "mail", mailprefix, 3);
		if (resourceId) {
			return {
				'id' : resourceId,
				'type' : 3
			};
		}

		var displayName = attendee.commonName;
		if (displayName) {
			FW_LOG("try to resolve resource by display name: " + displayName);
			resourceId = getParticipantIdByAttribute(connection, calendarId, "display_name", displayName, 3);
			if (resourceId) {
				return {
					'id' : resourceId,
					'type' : 3
				};
			}
		}

		FW_LOG("cannot resolve participants " + displayName + " add as external participant");
		return {
			'id' : 0,
			'display_name' : displayName,
			'type' : 5,
			'mail' : mail
		};
	} else {
		FW_WARN("unknown attendee role: " + attendee.role);
	}

	return undefined;
}

function addRecurrenceAttributes2Object(recurrenceItem, data) {
	FW_LOG("addRecurrenceAttributes2Object");

	var startDate = new Date(data['start_date']);

	var daysMapping = {
		'1' : 1,
		'2' : 2,
		'3' : 4,
		'4' : 8,
		'5' : 16,
		'6' : 32,
		'7' : 64
	};
	var recurrenceTypeMapping = {
		'DAILY' : 1,
		'WEEKLY' : 2,
		'MONTHLY' : 3,
		'YEARLY' : 4
	};

	if (recurrenceItem.type) {
		data['recurrence_type'] = recurrenceTypeMapping[recurrenceItem.type];
	}

	FW_LOG("recurrenceItem.type: " + recurrenceItem.type);

	if (recurrenceItem.type == 'WEEKLY') {
		var daysWeekly;
		var compByDayWeekly = recurrenceItem.getComponent("BYDAY", {});
		if (compByDayWeekly && String(compByDayWeekly).length > 0) {
			var dayArray = String(compByDayWeekly).split(",");
			daysWeekly = 0;

			if (dayArray.length == 0) {
				daysWeekly = parseInt(compByDayWeekly);
			} else {
				for each ( var day in dayArray) {
					daysWeekly += daysMapping[day];
				}
			}

			if (daysWeekly == 0) {
				daysWeekly = daysMapping[(startDate.getDay() + 1)];
			}
		} else {
			daysWeekly = daysMapping[(startDate.getDay() + 1)];
		}
		data['days'] = daysWeekly;
	} else if (recurrenceItem.type == 'MONTHLY' || recurrenceItem.type == 'YEARLY') {
		// check day_in_month
		var compByMonthDays = recurrenceItem.getComponent("BYMONTHDAY", {});
		if (compByMonthDays.length > 0) {
			FW_LOG("use compByMonthDays '" + compByMonthDays + "' for recurring appointment");
			data['day_in_month'] = compByMonthDays[0];
		} else {
			var days;
			var dayInMonth;
			var compByDay = recurrenceItem.getComponent("BYDAY", {});

			FW_LOG("use compByDay '" + compByDay + "' for recurring appointment");
			if (compByDay && String(compByDay).length > 0) {
				var daysString = String(compByDay);
				FW_LOG("daysString: " + daysString);
				if (daysString) {
					var daysInt = Number(daysString);
					var lastDay = false;

					if (daysInt < 0) {
						lastDay = true;

						// change negative int to positive int for later
						// calculation
						daysInt = daysInt - daysInt - daysInt;
					}

					FW_LOG("daysInt: " + daysInt);

					if (lastDay) {
						dayInMonth = -1;
					} else {
						dayInMonth = Number(daysInt / 8);

						// OX doesn't support the fifth day of the month so we
						// convert
						// it to the last day of the month
						if (dayInMonth == 5) {
							dayInMonth = -1;
						}
					}
					days = daysMapping[daysInt % 8];
				}
			} else {
				dayInMonth = startDate.getDate();
			}

			if (days) {
				data['days'] = days;
			}

			data['day_in_month'] = dayInMonth;
		}

		if (recurrenceItem.type == 'YEARLY') {
			var compByMonth = recurrenceItem.getComponent("BYMONTH", {});
			if (compByMonth && String(compByMonth).length > 0) {
				FW_LOG("compByMonth: " + compByMonth);
				if (compByMonth > 0) {
					data['month'] = compByMonth - 1;
				}
			} else {
				data['month'] = startDate.getMonth();
			}
		}
	}

	if (recurrenceItem.interval) {
		data['interval'] = recurrenceItem.interval;
	}

	try {

		var untilDateTime = recurrenceItem.untilDate;
		var untilDate = dateTime2Date(untilDateTime);
		data['until'] = untilDate.getTime();

	} catch (exc) {
	}

	try {
		if (recurrenceItem.count && recurrenceItem.count > 0) {
			data['occurrences'] = recurrenceItem.count;
		}
	} catch (exc) {
	}
}

function participants2Attendees(participants, participantDetails, createdBy, organizerId) {
	FW_LOG("participants2Attendees");
	var attendee_array = [];
	for ( var a = 0; a < participants.length; a++) {
		var participant = participants[a];
		var attendee = participant2Attendee(participant, participantDetails, createdBy, organizerId);
		if (attendee) {
			attendee_array[attendee_array.length] = attendee;
		}
	}

	return attendee_array;
}

function participant2Attendee(participant, participantDetails, createdBy, organizerId) {
	FW_LOG("participant2Attendee participant=" + participant.toSource());

	var type = participant['type'];
	var mail;
	var displayName;

	var attendee = createAttendee();

	if (type == '5') {
		displayName = participant['display_name'];
	} else {
		var id = participant['id'];
		var isOrganizer = (id == createdBy);

		if (isOrganizer) {
			FW_LOG("participant and created by are equals, add isOrganizer attribute");
			attendee.isOrganizer = true;
		}

		var participantDetail = participantDetails[id];
		if (!participantDetail) {
			FW_ERROR("unresolved participant found, id=" + id + " type=" + type);
			FW_LOG("participantDetails: " + participantDetails.toSource());
			return undefined;
		}

		displayName = participantDetail['display_name'];
	}

	if (type == '1' || type == '2') {
		// user participant
		mail = participantDetail['mail'];
		attendee.id = "mailto:" + mail;
		attendee.role = "REQ-PARTICIPANT";
	} else if (type == '3') {
		attendee.id = participantDetail['mail'];
		attendee.role = "CHAIR";
	} else if (type == '5') {
		mail = participant['mail'];
		attendee.id = "mailto:" + mail;
		attendee.role = "REQ-PARTICIPANT";
	}

	// ignore attendee if it is equals with the mail of the calendar account
	// and if the attendee is the organizer
	FW_LOG("organizerId=" + organizerId + ", attendee.id=" + attendee.id);
	if (organizerId == attendee.id && attendee.isOrganizer) {
		FW_LOG("ignore organizer attendee because it is equals with the attribute organizerId");
		return undefined;
	}

	attendee.commonName = displayName;
	attendee.userType = "INDIVIDUAL";
	attendee.makeImmutable();

	return attendee;
}

function object2item(item, event, attendee_array) {
	if (event.object_id) {
		event.id = event.object_id;
	}

	FW_LOG("try to parse object attributes for appointment with id: " + event.id);

	var fullTime = isFullTime(event['full_time']);

	var startDate = new Date(event['start_date']);
	var endDate = new Date(event['end_date']);

	item.startDate = date2dateTime(startDate, fullTime);
	FW_LOG("object2item item.startDate: " + item.startDate);
	item.endDate = date2dateTime(endDate, fullTime);
	FW_LOG("object2item item.endDate: " + item.endDate);

	var title = event['title'];
	var location = event['location'];
	var description = event['note'];

	// Parse summary (title)
	if (title) {
		item.title = title;
	}

	// Parse location
	if (location) {
		item.setProperty("LOCATION", location);
	} else {
		item.setProperty("LOCATION", "");
	}

	// Parse description
	if (description) {
		item.setProperty("DESCRIPTION", description);
	} else {
		item.setProperty("DESCRIPTION", "");
	}

	// Parse shown_as flag
	FW_LOG('object2item: shown_as=' + event['shown_as']);
	var shownAs = Number(event['shown_as']);
	if (shownAs >= 1 && shownAs <= 3) {
		item.setProperty("TRANSP", "OPAQUE");
	} else if (shownAs == 4) {
		item.setProperty("TRANSP", "TRANSPARENT");
	}

	var privateFlag = myParseBoolean(event['private_flag']);
	if (privateFlag) {
		item.privacy = "PRIVATE";
	} else {
		item.privacy = "PUBLIC";
	}

	// FW_LOG("parsed private flag: " + privateFlag + " to item.privacy: " +
	// item.privacy);

	for ( var a = 0; a < attendee_array.length; a++) {
		var attendee = attendee_array[a];
		if (!attendee.isOrganizer) {
			item.addAttendee(attendee);
		}
	}

	FW_LOG("parse common categories");

	var categoriesString = event['categories'];
	if (categoriesString) {
		categoriesString = String(categoriesString);
		var categoriesArray = categoriesString.split(",");
		item.setCategories(categoriesArray.length, categoriesArray);
	} else {
		item.setCategories(0, []);
	}

	FW_LOG("parse common recurring data");
	var recurrenceId = event['recurrence_id'];
	if (recurrenceId == event['object_id']) {
		if (!item.recurrenceInfo) {
			item.recurrenceInfo = createRecurrenceInfo(item);
		} else {
			item.recurrenceInfo.clearRecurrenceItems();
		}

		var recRule = data2RecurrenceRule(event);
		item.recurrenceInfo.appendRecurrenceItem(recRule);

		var changeExceptions = event['change_exceptions'];
		var deleteExceptions = event['delete_exceptions'];

		if (changeExceptions) {
			FW_LOG("changeExceptions: " + changeExceptions);
			removeRecurrenceExceptions(item, changeExceptions);
		}

		if (deleteExceptions) {
			FW_LOG("deleteExceptions: " + deleteExceptions);
			removeRecurrenceExceptions(item, deleteExceptions);
		}
	}

	FW_LOG("parse alarm");
	var alarm = event['alarm'];
	if (alarm) {
		alarm = Number(alarm);
		if (alarm >= 0) {

			item.clearAlarms();

			var mozillaAlarm = cal.createAlarm();
			mozillaAlarm.action = "DISPLAY";
			mozillaAlarm.related = Components.interfaces.calIAlarm.ALARM_RELATED_START;

			var alarmOffsetTB3 = cal.createDuration();
			alarmOffsetTB3.minutes = 0 - alarm;
			alarmOffsetTB3.normalize();

			mozillaAlarm.offset = alarmOffsetTB3;
			item.addAlarm(mozillaAlarm);

		}
	}

	FW_LOG("parsing common attributes finished");

	return item;
}

function data2RecurrenceRule(data) {
	FW_LOG("data2RecurrenceRule");

	var recurrenceItem = createRecurrenceRule().clone();

	var daysMapping = {
		1 : '1',
		2 : '2',
		4 : '3',
		8 : '4',
		16 : '5',
		32 : '6',
		64 : '7'
	};
	var recurrenceTypeMapping = {
		1 : 'DAILY',
		2 : 'WEEKLY',
		3 : 'MONTHLY',
		4 : 'YEARLY'
	};

	FW_LOG("try to read recurrence_type");

	if (data['recurrence_type']) {
		recurrenceItem.type = recurrenceTypeMapping[data['recurrence_type']];
		LOG("found recurrence type: " + recurrenceItem.type);
	}

	FW_LOG("try to read days");

	if (recurrenceItem.type == 'WEEKLY') {
		var days = parseInt(data['days']);
		var daysArray = [];
		var dayOrderArray = [ 64, 32, 16, 8, 4, 2, 1 ];
		for each ( var dayInt in dayOrderArray) {
			if (days >= dayInt) {
				daysArray[daysArray.length] = daysMapping[dayInt];
				days -= dayInt;
			}
		}
		recurrenceItem.setComponent("BYDAY", daysArray.length, daysArray);
	} else if ((recurrenceItem.type == 'MONTHLY') || (recurrenceItem.type == 'YEARLY')) {
		var dayInMonth = Number(data['day_in_month']);

		if (data['days']) {
			var oxdays = Number(data['days']);

			// check for alldays == 127
			if (oxdays == 127) {
				recurrenceItem.setComponent("BYMONTHDAY", 1, [ dayInMonth ]);
			} else {
				var tbdays;
				if (daysMapping[oxdays]) {
					tbdays = Number(daysMapping[oxdays]);
				} else {
					FW_WARN("unknown recurrence pattern 'days of week' for Thunderbird");
					tbdays = 1;
				}

				FW_LOG("convert 'days' from server: " + oxdays + " to 'days' from TB: " + tbdays);
				FW_LOG("use dayInMonth: " + dayInMonth + " for monthly calculation");

				var byday;
				if (dayInMonth == -1 || dayInMonth == 5) {
					// if dayInMonth is -1 or 5 set the byday value as a
					// negative integer
					// so it is the last day of the month from the given rule
					byday = -8 - tbdays;
				} else {
					byday = (dayInMonth * 8) + tbdays;
				}

				FW_LOG("set attribute BYDAY: " + byday + " calculated by days=" + tbdays + " and day_in_month=" + dayInMonth);
				recurrenceItem.setComponent("BYDAY", 1, [ byday ]);
			}
		} else {
			recurrenceItem.setComponent("BYMONTHDAY", 1, [ dayInMonth ]);
		}

		if (recurrenceItem.type == 'YEARLY') {
			FW_LOG("try to read month for recurring appointment");
			if (data['month']) {
				recurrenceItem.setComponent("BYMONTH", 1, [ data['month'] + 1 ]);
				FW_LOG("month found: " + data['month']);
			}
		}
	}

	FW_LOG("try to read interval");
	if (data['interval']) {
		recurrenceItem.interval = data['interval'];
		FW_LOG("interval found: " + data['interval']);
	}

	FW_LOG("try to read occurrences");
	if (data['occurrences']) {
		recurrenceItem.count = data['occurrences'];
		FW_LOG("occurrences found: " + data['occurrences']);
	}

	FW_LOG("try to read until");

	if (data['until']) {
		var MAX = 1893456000000;
		var untilAsLong = parseInt(data['until']);
		var endDateAsLong = parseInt(data['end_date']);
		FW_LOG("until found: " + untilAsLong);
		if (untilAsLong > MAX) { // 01.01.2020
			untilAsLong = MAX;
		}
		var endDate = new Date(endDateAsLong);
		var untilDate = new Date(untilAsLong);

		FW_LOG("parse until (end date): " + endDate);

		untilDate.setHours(endDate.getHours());
		untilDate.setMinutes(endDate.getMinutes());

		var untilDateTime = date2dateTime(untilDate, false);

		FW_LOG("parsed until: " + untilAsLong + " to date time: " + untilDateTime);

		recurrenceItem.untilDate = untilDateTime;

	}

	return recurrenceItem;
}

function getPrefRangeStartAsLong(connection) {
	var calendarRangeAsInt = getPrefRangeAsInt(connection, "calendar-range-start");
	if (calendarRangeAsInt) {
		var monthInMillis = 30 * 24 * 60 * 60 * 1000;
		var calendarRange = date2utc_long(new Date()) - (calendarRangeAsInt * monthInMillis);
		return calendarRange;
	}

	return undefined;
}

function getPrefRangeEndAsLong(connection) {
	var calendarRangeAsInt = getPrefRangeAsInt(connection, "calendar-range-end");
	if (calendarRangeAsInt) {
		var monthInMillis = 30 * 24 * 60 * 60 * 1000;
		var calendarRange = date2utc_long(new Date()) + (calendarRangeAsInt * monthInMillis);
		return calendarRange;
	}

	return undefined;
}

function getPrefRangeAsInt(connection, rangeName) {
	var calendarRange = getOXCalendarPref(connection, rangeName);
	if (!calendarRange) {
		calendarRange = "6";
	}

	calendarRange = parseInt(calendarRange);

	if (calendarRange > 0) {
		return calendarRange;
	}

	return undefined;
}

/**
 * This method removed the given exceptions from the recurrenceInfo object
 * 
 * @param {Object}
 *            item - calItemBase with the recurrenceInfo attribute
 * @param {Object}
 *            exceptions - an array with long values
 */

function removeRecurrenceExceptions(item, exceptions) {
	FW_LOG("removeRecurrenceExceptions: " + exceptions);
	// removeOccurrenceAt (in calIDateTime aRecurrenceId)
	if (exceptions) {
		FW_LOG("exceptions.length: " + exceptions.length);
		for ( var a = 0; a < exceptions.length; a++) {
			var date = new Date(exceptions[a]);

			var removeOccurrenceDate = item.startDate.clone();
			removeOccurrenceDate.year = date.getUTCFullYear();
			removeOccurrenceDate.month = date.getUTCMonth();
			removeOccurrenceDate.day = date.getUTCDate();

			item.recurrenceInfo.removeOccurrenceAt(removeOccurrenceDate);
		}
	}
}

/**
 * isFullTime return true or false depending on the given parameter If the
 * parameter is undefinied the return value is false If the parameter is set a
 * string or a boolean value is checked
 */
function isFullTime(fullTime) {
	if (fullTime) {
		var stringFullTime = String(fullTime);
		if (stringFullTime == "true") {
			return true;
		}
	}
	return false;
}

/**
 * dateTime2Date Convert a calIDateTime to a javascript date object
 * 
 * @param dateTime
 *            A calIDateTime object
 * @return A javascript date object
 */
function dateTime2Date(dateTime) {
	var localDateTime = dateTime.getInTimezone(UTC());

	var date = new Date(0);

	// correct order of setting is very essential here !
	// otherwise we might get into ugly rounding troubles on 31th day... believe
	// me
	// we could also have used e.g. "var date = new Date(0);" see above
	date.setUTCFullYear(localDateTime.year);
	date.setUTCMonth(localDateTime.month);
	date.setUTCDate(localDateTime.day);

	if (!dateTime.isDate) {
		date.setUTCHours(localDateTime.hour);
		date.setUTCMinutes(localDateTime.minute);
		date.setUTCSeconds(localDateTime.second);
	} else {
		date.setUTCHours(0);
		date.setUTCMinutes(0);
		date.setUTCSeconds(0);
	}

	date.setUTCMilliseconds(0);
	return date;
}

/**
 * date2dateTime Convert a javascript date object to calIDateTime
 * 
 * @param date
 *            A javascript date object
 * @param fullTime
 *            is true if date has not time values
 * @return A calIDateTime object
 */
function date2dateTime(date, fullTime) {
	var localDateTime = cal.jsDateToDateTime(date, calendarDefaultTimezone());
	localDateTime.isDate = fullTime;

	/*
	 * var dateTime = createDateTime(); localDateTime =
	 * dateTime.getInTimezone(UTC());
	 * 
	 * localDateTime.isDate = fullTime;
	 * 
	 * localDateTime.year = date.getUTCFullYear(); localDateTime.month =
	 * date.getUTCMonth(); localDateTime.day = date.getUTCDate();
	 * 
	 * if (!localDateTime.isDate) { localDateTime.hour = date.getUTCHours();
	 * localDateTime.minute = date.getUTCMinutes(); localDateTime.second =
	 * date.getUTCSeconds(); }
	 */

	FW_LOG("date2dateTime date=" + date + ", fullTime=" + fullTime + ", localDateTime=" + localDateTime);
	return localDateTime;
}

function date2long(date) {
	return Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(), date.getSeconds());
}

function date2utc_long(date) {
	return Date.UTC(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
}

function parseStartAndEndFromAppointment(appointmentObj) {
	var startDate = appointmentObj['start_date'];
	var endDate = appointmentObj['end_date'];

	var recurrenceType = appointmentObj['recurrence_type'];

	if (recurrenceType && recurrenceType > 0) {
		var until = appointmentObj['until'];
		if (until) {
			endDate = appointmentObj['until'];
		} else {
			// set enddate to max recurrence end date 01.01.2010
			endDate = 1893456000000;
		}
	}

	var start = Number(startDate) - (12 * 60 * 60 * 1000);
	var end = Number(endDate) + (12 * 60 * 60 * 1000);

	return [ start, end ];
}

/*
 * This LOGGING methods were orginal taken from the google calender provider but
 * a little modified by me
 */
function LOGitem(item) {
	var attendees = item.getAttendees({});
	var attendeeString = "";
	for ( var a in attendees) {
		attendeeString += "\n" + LOGattendee(a);
	}

	var rstr = "\n";
	if (item.recurrenceInfo) {
		var ritems = item.recurrenceInfo.getRecurrenceItems({});
		for each ( var ritem in ritems) {
			rstr += "\t\t" + ritem.icalProperty.icalString;
		}

		rstr += "\tExceptions:\n";
		var exids = item.recurrenceInfo.getExceptionIds({});
		for ( var exc in exids) {
			rstr += "\t\t" + exc + "\n";
		}
	}

	FW_LOG("Logging calIEvent:" + "\n\tid:" + item.id + "\n\ttitle:" + item.title + "\n\tcontent:" + item.getProperty("DESCRIPTION") + "\n\ttransparency:"
			+ item.getProperty("TRANSP") + "\n\tstatus:" + item.status + "\n\tstartTime:" + item.startDate.toString() + "\n\tendTime:"
			+ item.endDate.toString() + "\n\tlocation:" + item.getProperty("LOCATION") + "\n\tprivacy:" + item.privacy + "\n\talarmOffset:" + item.alarmOffset
			+ "\n\talarmLastAck:" + item.alarmLastAck + "\n\tisOccurrence: " + (item.recurrenceId != null) + "\n\trecurrenceId: " + item.recurrenceId
			+ "\n\tOrganizer: " + LOGattendee(item.organizer) + "\n\tAttendees: " + attendeeString + "\n\trecurrence: "
			+ (rstr.length > 1 ? "yes: " + rstr : "no"));
}

function LOGattendee(aAttendee, asString) {
	return aAttendee
			&& ("\n\t\tID: " + aAttendee.id + "\n\t\t\tName: " + aAttendee.commonName + "\n\t\t\tRsvp: " + aAttendee.rsvp + "\n\t\t\tIs Organizer: "
					+ (aAttendee.isOrganizer ? "yes" : "no") + "\n\t\t\tRole: " + aAttendee.role + "\n\t\t\tStatus: " + aAttendee.participationStatus);
}

function LOGcalIFreeBusyInterval(calFreeBusyInterval) {
	var logVal = "calIFreeBusyInterval:\n";
	logVal += "\tcalId: " + calFreeBusyInterval.calId + "\n";
	logVal += "\tPeriod: \n" + LOGcalIPeriod(calFreeBusyInterval.interval);
	logVal += "\tfreeBusyType: " + calFreeBusyInterval.freeBusyType + "\n";

	return logVal;
}

function LOGcalIPeriod(calPeriod) {
	var logVal = "\t\tStart Date Time: " + calPeriod.start + "\n";
	logVal += "\t\tEnd Date Time: " + calPeriod.end + "\n";
	logVal += "\t\tDuration " + calPeriod.duration + "\n";

	return logVal;
}
